/* PIC32 DDS Embedded Controller                                            */
/* 03/11/13	version ver 0.1                                             */
/* 23/12/13 all going but AM not implemented yet...                         */
/*                                                                          */
/*                                                                          */
/* Go back and check if you want Interpolated output to be long long or long */
/*                                                                          */
/*                                                                          */
/* V1.1 Increased sample rate to 64KHz                                      */
/* Numerous tweaks to the HMI                                               */


#include <time.h>
#include "xc.h"
#include <plib.h>
#include "delay.h"
#include "dig_cross.h"
#include "AD1940.h"
#include <p32mx450f256h.h>
#include "stdio.h"
#include "EEPROM.h"
#include <peripheral/int.h>
#include <peripheral/ports.h>
#include "Sine_Lookup.h"


// Configuration Bit settings
// SYSCLK = 80 MHz (8MHz Crystal/ FPLLIDIV * FPLLMUL / FPLLODIV)
// PBCLK = 10 MHz
// Primary Osc w/PLL (XT+,HS+,EC+PLL)
// WDT OFF
// Other options are don't care
//
#pragma config FPLLMUL = MUL_20, FPLLIDIV = DIV_2, FPLLODIV = DIV_1, FWDTEN = OFF
#pragma config POSCMOD = HS, FNOSC = PRIPLL, FPBDIV = DIV_1
#pragma config UPLLIDIV = DIV_2, UPLLEN = ON
#pragma config JTAGEN = OFF
#pragma config FSRSSEL =  PRIORITY_3


static unsigned char Old_Rot_Status = 0;  			/* Records state of rotary control */
int Fast_Loop_Counter = Fast_Counter_Init; 	/* Used to set Fast_Cnt */
int Slow_Loop_Counter = Slow_Counter_Init; 	/* Used to see if user turning control continuously */
static unsigned char Keypress_Read = 0;			/* this is the data on key / rotation changes */
static unsigned char Key_Just_Pressed = 0;  		/* use this to keep track of which keys have been pressed */
int Fast_Change = 1;			/* use this to indicate fast change of values */

data_val_struct Data_Values;                            /* This is where all the data is stored... */
char loop, test_temp;

/* DDS data storage */
unsigned int Phase_Reg_Temp, Next_Phase_Reg_Temp;/* working variable */
unsigned int Phase_Acc_Ch1, Phase_Acc_Ch2;       /* Channel 1 and 2 phase accumulators */
unsigned int Next_Phase_Acc_Ch1, Next_Phase_Acc_Ch2; /* Channel 1 and 2 lookahead phase accumulators */
unsigned int Delta_Phase_Acc_Ch1, Delta_Phase_Acc_Ch2; /* Channel 1 and 2 lookahead phase accumulators */
unsigned int Old_Phase_Acc_Ch1, Old_Phase_Acc_Ch2;   /* Channel 1 and 2 lookback phase accumulators */
int Ch1_Level_Mult, Ch2_Level_Mult;     /* channel 1 and 2 level multiplier */
long long Interpolated_Output;           /* calculations for interpolation requires long int */
long long Temp_Interpolated_Output;           /* calculations for interpolation requires long int */
int Phase_Inc_Ch1, Phase_Inc_Ch2;       /* Channel 1 and 2 phase increment per clock */
int Cycles_On_Ch1, Cycles_On_Ch2;       /* Channel 1 and 2 number of cycles on for pulse mode */
int Pulse_On_Ch1, Pulse_On_Ch2;         /* pulse train on? */
int Cycles_Off_Ch1, Cycles_Off_Ch2;     /* Channel 1 and 2 number of cycles off for pulse mode */
int FM_Phase_Acc_Ch1, FM_Phase_Acc_Ch2; /* Channel 1 and 2 phase acc for FM */
int AM_Phase_Acc_Ch1, AM_Phase_Acc_Ch2; /* Channel 1 and 2 phase acc for AM */
int Ch1_Sweep_Upwards, Ch2_Sweep_Upwards; /* is the sweep up or down? */
long long Ch1_Sweep_End, Ch2_Sweep_End;       /* Channel 1 and 2 sweep end freq */
long long Ch1_Sweep_Phase_Inc, Ch2_Sweep_Phase_Inc; /* Channel 1 and 2 sweep phase increment */
long long Ch1_Sweep_Phase_Reg, Ch2_Sweep_Phase_Reg; /* Channel 1 and 2 sweep phase increment */
int Ch1_Sweep_On, Ch2_Sweep_On;         /* Channel 1 and 2 sweep on */
int Ch1_Pulse_Active, Ch2_Pulse_Active; /* Channel 1 and 2 Pule on */
int Ch1_Pulse_Output_Active, Ch2_Pulse_Output_Active; /* Channel 1 and 2 Pule Output on */
int Ch1_Pulse_Cycles_Active, Ch2_Pulse_Cycles_Active; /* Channel 1 and 2 Pule Output active */
int Ch1_Pulse_Cycles_InActive, Ch2_Pulse_Cycles_InActive; /* Channel 1 and 2 Pule Output inactive */
int Ch1_Pulse_Counter, Ch2_Pulse_Counter;           /* Channel 1 and 2 Pule counters */
int Ch1_Phase_Delta, Ch2_Phase_Delta;               /* Channel 1 and 2 Phase offsets for addition to output phase */
int Ch1_LUT[4096], Ch2_LUT[4096];       /* channel 1 and 2 waveform LUTs */
/* DDS and waveform stuff for AM and FM */
int Ch1_FM_LUT[2048], Ch2_FM_LUT[2048]; /* channel 1 and 2 PM waveform LUTs */
unsigned int Ch1_AM_Phase_Reg, Ch2_AM_Phase_Reg; /* Channel 1 and 2 AM DDS Accumulator*/
int Ch1_FM_Phase_Offset, Ch2_FM_Phase_Offset;    /* Channel 1 and 2 offsets for Main DDS Phase Increment*/
unsigned int Ch1_FM_Phase_Reg, Ch2_FM_Phase_Reg; /* Channel 1 and 2 FM DDS Accumulator*/
unsigned int Ch1_AM_Phase_Inc, Ch2_AM_Phase_Inc; /* Channel 1 and 2 AM DDS Phase invrement*/
unsigned int Ch1_FM_Phase_Inc, Ch2_FM_Phase_Inc; /* Channel 1 and 2 FM DDS Phase invrement*/
int Ch1_AM_Mult, Ch2_AM_Mult;                   /* use this to "multuply" the sine lookup (scale 1-100% of AM */
int Ch1_AM_On, Ch2_AM_On;                       /* flags */
int Ch1_FM_On, Ch2_FM_On;                       /* flags */
long long Ch1_AM_Temp, Ch2_AM_Temp;             /* intermediate calc for AM */

/* You might want to move these into the display LCD function - though it is really not a gut buster for these two */
char line1[] =   Line1_Init;
char line2[] =   Line2_Init;
char *line1_ptr, *line2_ptr;				/* use these to point into the arrays */

/* These are ROM constants used in bnuilding the display */
const char Blank_Char = Blank_Char_Init;            /* ROM */
const char Sel_Str[] = Line2_Select_Init;           /* ROM */
unsigned int display_temp;
const char Ch1_Str[] =  Line1_Channel_1_Init;       /* ROM */
const char Ch2_Str[] =  Line1_Channel_2_Init;       /* ROM */
const char Save_Str[] =  Line1_Save_Init;           /* ROM */
const char Erase_Str1[] =  Line1_Erase_Check;       /* ROM */
const char Erase_Str2_Yes[] =  Line2_Erase_Check_Y; /* ROM */
const char Erase_Str2_No[] =  Line2_Erase_Check_N;  /* ROM */
const char Load_Str[] =  Line1_Load_Init;           /* ROM */
const char Choose_Str[] = Line1_Choose_Str_Init;    /* ROM */
const char Vol_Str[] = Line1_Level_Init;            /* ROM */
const char Vol2_Str[] = Line2_Level_Init;           /* ROM */

const char Channel_Setup_Choice_Display[8][17] =
{
"Frequency       ",
"Attenuation     ",
"Waveform        ",
"Sweep           ",
"Channel Phase   ",
"Amplitude Mod   ",
"Frequency Mod   ",
"Pulse Modulation"
};						/* ROM */

const char Setup_Choice_Units_Display[8][17] =
{
"        Hz      ",   /* Freq */
"      dB        ",   /* Atten */
"                ",   /* Waveform Various*/
"                ",   /* Sweep Various*/
"      Deg       ",   /* phase */
"                ",   /* AM On or Off*/
"                ",   /* FM On or Off*/
"                "    /* Pulse Mod On or Off*/
};						/* ROM */

const char Choice_Display[4][17] =
{
"Channel 1       ",
"Channel 2       ",
"Save Parameters?",
"Load Parameters?",
};						/* ROM */


const char Waveform_Select_Display[5][17] =
{
"Sine            ",
"Triangle        ",
"Square          ",
"Pulse           ",
"Noise           "
};						/* ROM */


const char Freq_Stepsize_Select_Display[2][17] =
{
"Freq Stepsize   ",
"      Hz        "
};						/* ROM */


const char Ratio_Select_Display[2][17] =
{
"Mark-Space Ratio",
"   %            "
};						/* ROM */

const char AM_Mod_Select_Display[2][17] =
{
"AM Level        ",
"   %            "
};						/* ROM */

const char Sweep_Select_Display[2][17] =
{
"No Sweep        ",
"Linear          "
};						/* ROM */

const char Setup_Cycles_On_Display[2][17] =
{
"Pulse Train:    ",
"     Cycles On  "
};						/* ROM */

const char Setup_Cycles_Off_Display[2][17] =
{
"Pulse Train:    ",
"     Cycles Off "
};						/* ROM */

const char Sweep_Freq_Select_Display[2][17] =
{
"Sweep Stop Freq ",
"      Hz        "
};						/* ROM */

const char Sweep_Freq_Time_Display[2][17] =
{
"Sweep Time      ",
"     Seconds    "
};						/* ROM */

const char AM_Freq_Select_Display[2][17] =
{
"AM Frequency    ",
"     Hz         "
};						/* ROM */

const char FM_Mod_Select_Display[2][17] =
{
"FM Excursion    ",
"     Hz         "
};						/* ROM */

const char FM_Freq_Select_Display[2][17] =
{
"FM Frequency    ",
"     Hz         "
};						/* ROM */

const char FM_Select_Display[2][17] =
{
"FM Excursion    ",
"FM Frequency    "
};						/* ROM */

const char Pulse_Select_Display[2][17] =
{
"Cycles On       ",
"Cycles Off      "
};						/* ROM */

const char Yes_No[2][4] =
{
"Yes",
"No "
};


/****************************************************************************/
/* Initialise the PIC							    */
/****************************************************************************/
void Init_IO(void)
{

    int spi_con1, spi_con2;

/* clear IO LOck bit */
    CFGCON = CFGCON  & (0xFFFF - 0x2000);

    
/* We will eventually want to be able to set the following: */
/* IEC0 bit 4, Timer1 */
/* IEC0 bit 9, Timer2 */
/* IEC1 bit 2, USB */
/* IEC1 bit 5, SPI1TX */
//    IEC0 = 0x0000;		// Disable interrupt register 0
//    IEC1 = 0x0000;		// Disable interrupt REGISTER 1
//    IEC2 = 0x0000;		// Disable interrupt REGISTER 2

/* ADCON setup */
    ANSELB = 0x0000;            // A/D Port reconfiguration - dig
    ANSELC = 0x0000;		// A/D Port reconfiguration - dig
    ANSELD = 0x0000;            // A/D Port reconfiguration - dig
    ANSELE = 0x0000;		// A/D Port reconfiguration - dig
    ANSELF = 0x0000;		// A/D Port reconfiguration - dig
    ANSELG = 0x0000;            // A/D Port reconfiguration - dig

/* set peripheral pin select pins*/
/* there are lits of these.      */

/* B0 = PGED1*/
/* B1 = PGEC1*/
/* B2  = REFCLKO */
PPSOutput(3,RPB2,REFCLKO); // Set RB2 pin as output for REFCLK
/* B3  =SPARE */
/* B4  =SPARE */
/* B5  =SPARE */
/* B6  Rotary_Encoder_Select_0 (input)*/
/* B7  Rotary_Encoder_Select_1 (input)*/

/* RB8 - LCD0 (output) */
/* RB9 - LCD1 (output) */
/* RB10 - LCD2 (output) */
//RPB10R = 0x0000;
/* RB11 - LCD3 (output) */
/* RB12 - LCD4 (output) */
/* RB13 - LCD5 (output) */
/* RB14 - LCD6 (output) */
/* RB15 - LCD7 (output) */

/* RC13 = OSCILLATOR */
/* RC14 = OSCILLATOR */
/* RC12 = OSCILLATOR */
/* RC15 = OSCILLATOR */

/* RD0 = SPARE*/
/* RD1 = SPI_CS1*/
/* RD2 = SPI_SCK1*/
/* RD3 = SPI_SDI1 = "0000" = input*/
//SDI1R = 0x0000;
    PPSInput(2,RPD3,SDI1);   //Set RD3 as input for SDI1
    PPSOutput(3,RPD4,SDO1); // Set RD4 pin as output for SDO1

/* RD5 = LCD_RW (output)*/
/* RD6 = LCD_RS (output)*/
/* RD7 = LCD_CS (output)*/

/* RD8 = DSP_RESET*/
/* RD9 = SPI_SS1 - "0111"*/
    PPSOutput(3,RPD9,SS1); // Set RD4 pin as output for SS1

/* RD10 = ANALOGUE_CD*/
/* RD11 = UNUSED (output)*/

/* RE0 = Drives logic 1 as output for test board. */
/* RE1 = MODE_SEL (input)*/
/* RE2 = FN_SEL (input)*/
/* RE3 = PHASEA (input)*/
/* RE4 = PHASEB (input)*/
/* RE5 = SPARE*/
/* RE6 = SPARE*/
/* RE7 = SPARE*/

/* RF0 EEPROM CS\ - active low (output)*/
/* RF1 EEPROM Hold\ - active low (output)*/
/* RF4 UNUSED (output)*/
/* RF5 UNUSED (output)*/

/* RG6 = SPI_SCK2 */
//    PPSOutput(3,RPG6,REFCLKO); // Set RG6 pin as output for REFCLKO

/* RG7 = SPI_SDI2 "0001" */
/* RG8 = SPI_SDO2 = "0110" */
    PPSInput(2,RPG7,SDI2);   //Set RD3 as input for SDI2
    PPSOutput(1,RPG8,SDO2); // Set RD4 pin as output for SDO2

/* RG9 = SPI_SS2 = "0001" */
    PPSOutput(4,RPG9,SS2); // Set RG9 pin as output for SS2

/* initialise IO */
/* B6,7 = input remainder = output */
/* C = output */
/* D3 = input 0x0008*/
/* E1,2,3,4 = input 0x001E*/
/* F = output */
/* G7 = input */
        TRISB=0X00C0;
        TRISC=0X0000;
        TRISD=0X0008;
        TRISE=0x001E;
        TRISF=0x0000;
        TRISG=0x0080;

/* Set outputs to normal logic */
        ODCB = 0x0000;
        ODCC = 0x0000;
        ODCD = 0x0000;
        ODCE = 0x0000;
        ODCF = 0x0000;
        ODCG = 0x0000;

/* Write to ports */
/* need to update these to appropriate values */
        PORTB = 0x0000;
        PORTC = 0x0000;
        PORTD = 0x0000;
        PORTE = 0x0001;  /* drive logic high to E0*/
        PORTF = 0x0000;
        PORTG = 0x0000;

/* Change Notice Enable = OFF*/
        CNCONB = 0X0000;
        CNCONC = 0X0000;
        CNCOND = 0X0000;
        CNCONE = 0X0000;
        CNCONF = 0X0000;
        CNCONG = 0X0000;


/* Hold DSP in RESET */
SetDSPReset;        /* Hold DSP Reset Low  */
SetDACReset;	    /* Hold DAC in RESET */
PORTB = 0;   
CselHighPGA2320;
CselClearEEPROM; /* Clear EEPROM Chip Select*/
HoldClearEEPROM; /* Clear EEPROM hold signal */

/* Initialise the SPI interface */
/*
 * Channel 1
 * Master
 * Positive Pol
 * No slave enable
 * Default clock edge
 * Sample phase control
 * 8 bit character mode (dont use 16 or 32)
 * SDO pin enable (default)
 * No idle functionality
 * No debug
 * Turn SPI On
 * No frame
 *
 * fpbDiv 64
*/
SpiChnOpen(1,SPICON_MSTEN | SPICON_CKP | SPICON_ON, 256);

/* Configure Reference Clock Output to 12.288MHz.  */
mOSCREFOTRIMSet(REFTRIM);
OSCREFConfig(OSC_REFOCON_USBPLL, //USB-PLL clock output used as REFCLKO source
            OSC_REFOCON_OE | OSC_REFOCON_ON, //Enable and turn on the REFCLKO
            RODIV_PROG);

 //Configure SPI in I2S mode with 24-bit stereo audio.
 spi_con1 = SPI_OPEN_MSTEN | //Master mode enable
        SPI_OPEN_SSEN | //Enable slave select function
        SPI_OPEN_CKP_HIGH | //Clock polarity Idle High Actie Low
        SPI_OPEN_ENHBUF | // Enhanced Mode
        SPI_OPEN_TBE_HALF_EMPTY | //generate event when TX buffer more than half empty
        SPI_OPEN_MODE16 | //Data mode: 24b
        SPI_OPEN_MODE32 | //Data mode: 24b
        SPI_OPEN_MCLKSEL | //Clock selected is reference clock
        SPI_OPEN_FSP_HIGH; //Frame Sync Pulse is active high

 spi_con2 = SPI_OPEN2_AUDEN | //Enable Audio mode
        SPI_OPEN2_AUDMOD_I2S; //Enable I2S mode

//Configure and turn on the SPI1 module.
SpiChnOpenEx(SPI_CHANNEL2, spi_con1, spi_con2, (mclk / baud_rate));

}



/****************************************************************************/
/* Do this in one place  -  save memory and hassles with consistency!    */
/****************************************************************************/
void Load_From_Memory(char Mem_Bank)
{
unsigned char HighAdd, LowAdd;
int Data_Addr;          /* temp address counter */

/* Select EEPROM device */
CselEEPROM;
/* Read in the parameters for crossover points from default bank of EEPROM */

/* This is the base address of the channel's data in EEPROM*/
Data_Addr = Mem_Bank * ParmSet_Array_Size;

/* CH_1_Freq Freq_Sel at Freq_Offset */
HDWordReadSPI((Data_Addr + Freq_Offset), &(Data_Values.Ch_1_Freq_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Freq_Sel < Freq_Min)
    Data_Values.Ch_1_Freq_Sel = Freq_Min;
else if (Data_Values.Ch_1_Freq_Sel > Freq_Max)
    Data_Values.Ch_1_Freq_Sel = Freq_Max;

/* Ch_1_Level_Sel at Level_Offset */
HDWordReadSPI((Data_Addr + Level_Offset), &(Data_Values.Ch_1_Level_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Level_Sel < Level_Min)
    Data_Values.Ch_1_Level_Sel = Level_Min;
else if (Data_Values.Ch_1_Level_Sel > Level_Max)
    Data_Values.Ch_1_Level_Sel = Level_Max;

/* Ch_1_Waveform_Sel at Waveform_Offset*/
HDWordReadSPI((Data_Addr + Waveform_Offset), &(Data_Values.Ch_1_Waveform_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Waveform_Sel < Waveform_Min)
    Data_Values.Ch_1_Waveform_Sel = Waveform_Min;
else if (Data_Values.Ch_1_Waveform_Sel > Waveform_Max)
    Data_Values.Ch_1_Waveform_Sel = Waveform_Max;

/* Ch_1_Ratio_Sel at Ratio_Offset*/
HDWordReadSPI((Data_Addr + Ratio_Offset), &(Data_Values.Ch_1_Ratio_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Ratio_Sel < Ratio_Min)
    Data_Values.Ch_1_Ratio_Sel = Ratio_Min;
else if (Data_Values.Ch_1_Ratio_Sel > Ratio_Max)
    Data_Values.Ch_1_Ratio_Sel = Ratio_Max;

/* Ch_1_Sweep_Stop_Freq_Sel at Sweep_Stop_Freq_Offset*/
HDWordReadSPI((Data_Addr + Sweep_Stop_Freq_Offset), &(Data_Values.Ch_1_Sweep_Stop_Freq_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Sweep_Stop_Freq_Sel < Sweep_Stop_Min)
    Data_Values.Ch_1_Sweep_Stop_Freq_Sel = Sweep_Stop_Min;
else if (Data_Values.Ch_1_Sweep_Stop_Freq_Sel > Sweep_Stop_Max)
    Data_Values.Ch_1_Sweep_Stop_Freq_Sel = Sweep_Stop_Max;

/* Ch_1_Sweep_Type_Sel at Sweep_Type_Offset*/
HDWordReadSPI((Data_Addr + Sweep_Type_Offset), &(Data_Values.Ch_1_Sweep_Type_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Sweep_Type_Sel < Sweep_Type_Min)
    Data_Values.Ch_1_Sweep_Type_Sel = Sweep_Type_Min;
else if (Data_Values.Ch_1_Sweep_Type_Sel > Sweep_Type_Max)
    Data_Values.Ch_1_Sweep_Type_Sel = Sweep_Type_Max;

/* Ch_1_Sweep_Time_Sel at Sweep_Type_Offset*/
HDWordReadSPI((Data_Addr + Sweep_Time_Offset), &(Data_Values.Ch_1_Sweep_Time_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Sweep_Time_Sel < Sweep_Time_Min)
    Data_Values.Ch_1_Sweep_Time_Sel = Sweep_Time_Min;
else if (Data_Values.Ch_1_Sweep_Time_Sel > Sweep_Time_Max)
    Data_Values.Ch_1_Sweep_Time_Sel = Sweep_Time_Max;

/* Ch_1_Phase_Sel at Phase_Offset*/
HDWordReadSPI((Data_Addr + Phase_Offset), &(Data_Values.Ch_1_Phase_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Phase_Sel < Phase_Min)
    Data_Values.Ch_1_Phase_Sel = Phase_Min;
else if (Data_Values.Ch_1_Phase_Sel > Phase_Max)
    Data_Values.Ch_1_Phase_Sel = Phase_Max;

/* Ch_1_AM_Sel at AM_Offset*/
HDWordReadSPI((Data_Addr + AM_Offset), &(Data_Values.Ch_1_AM_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_AM_Sel < AM_On_Min)
    Data_Values.Ch_1_AM_Sel = AM_On_Min;
else if (Data_Values.Ch_1_AM_Sel > AM_On_Max)
    Data_Values.Ch_1_AM_Sel = AM_On_Max;

/* Ch_1_FM_Sel at FM_Offset*/
HDWordReadSPI((Data_Addr + FM_Offset), &(Data_Values.Ch_1_FM_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_FM_Sel < FM_On_Min)
    Data_Values.Ch_1_FM_Sel = FM_On_Min;
else if (Data_Values.Ch_1_FM_Sel > FM_On_Max)
    Data_Values.Ch_1_FM_Sel = FM_On_Max;

/* Ch_1_Pulse_Sel at Pulse_Offset*/
HDWordReadSPI((Data_Addr + Pulse_Offset), &(Data_Values.Ch_1_Pulse_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_Pulse_Sel < Pulse_On_Min)
    Data_Values.Ch_1_Pulse_Sel = Pulse_On_Min;
else if (Data_Values.Ch_1_Pulse_Sel > Pulse_On_Max)
    Data_Values.Ch_1_Pulse_Sel = Pulse_On_Max;

/* Ch_1_AM_Level_Sel at AM_Level_Offset*/
HDWordReadSPI((Data_Addr + AM_Level_Offset), &(Data_Values.Ch_1_AM_Level_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_AM_Level_Sel < AM_Depth_Min)
    Data_Values.Ch_1_AM_Level_Sel = AM_Depth_Min;
else if (Data_Values.Ch_1_AM_Level_Sel > AM_Depth_Max)
    Data_Values.Ch_1_AM_Level_Sel = AM_Depth_Max;

/* Ch_1_AM_Freq_Sel at AM_Freq_Offset*/
HDWordReadSPI((Data_Addr + AM_Freq_Offset), &(Data_Values.Ch_1_AM_Freq_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_AM_Freq_Sel < AM_Freq_Min)
    Data_Values.Ch_1_AM_Freq_Sel = AM_Freq_Min;
else if (Data_Values.Ch_1_AM_Freq_Sel > AM_Freq_Max)
    Data_Values.Ch_1_AM_Freq_Sel = AM_Freq_Max;

/* Ch_1_FM_Freq_Sel at FM_Freq_Offset*/
HDWordReadSPI((Data_Addr + FM_Freq_Offset), &(Data_Values.Ch_1_FM_Freq_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_FM_Freq_Sel < FM_Freq_Min)
    Data_Values.Ch_1_FM_Freq_Sel = FM_Freq_Min;
else if (Data_Values.Ch_1_FM_Freq_Sel > FM_Freq_Max)
    Data_Values.Ch_1_FM_Freq_Sel = FM_Freq_Max;

/* Ch_1_FM_Escursion_Sel at FM_Excursion_Offset*/
HDWordReadSPI((Data_Addr + FM_Excursion_Offset), &(Data_Values.Ch_1_FM_Excursion_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_FM_Excursion_Sel < FM_Excursion_Min)
    Data_Values.Ch_1_FM_Excursion_Sel = FM_Excursion_Min;
else if (Data_Values.Ch_1_FM_Excursion_Sel > FM_Excursion_Max)
    Data_Values.Ch_1_FM_Excursion_Sel = FM_Excursion_Max;

/* Ch_1_PM_Cycles_On_Sel at Cycles_On_Offset*/
HDWordReadSPI((Data_Addr + Cycles_On_Offset), &(Data_Values.Ch_1_PM_Cycles_On_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_PM_Cycles_On_Sel < PM_Cycles_On_Min)
    Data_Values.Ch_1_PM_Cycles_On_Sel = PM_Cycles_On_Min;
else if (Data_Values.Ch_1_PM_Cycles_On_Sel > PM_Cycles_On_Max)
    Data_Values.Ch_1_PM_Cycles_On_Sel = PM_Cycles_On_Max;

/* Ch_1_PM_Cycles_Off_Sel at Cycles_Of_Offset*/
HDWordReadSPI((Data_Addr + Cycles_Off_Offset), &(Data_Values.Ch_1_PM_Cycles_Off_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_1_PM_Cycles_Off_Sel < PM_Cycles_Off_Min)
    Data_Values.Ch_1_PM_Cycles_Off_Sel = PM_Cycles_Off_Min;
else if (Data_Values.Ch_1_PM_Cycles_Off_Sel > PM_Cycles_Off_Max)
    Data_Values.Ch_1_PM_Cycles_Off_Sel = PM_Cycles_Off_Max;



/* This is the base address of the second channel's data in EEPROM*/
Data_Addr = Mem_Bank * ParmSet_Array_Size + Channel_Set_Array_Size;

/* CH_2_Freq Freq_Sel at Freq_Offset */
HDWordReadSPI((Data_Addr + Freq_Offset), &(Data_Values.Ch_2_Freq_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Freq_Sel < Freq_Min)
    Data_Values.Ch_2_Freq_Sel = Freq_Min;
else if (Data_Values.Ch_2_Freq_Sel > Freq_Max)
    Data_Values.Ch_2_Freq_Sel = Freq_Max;

/* Ch_2_Level_Sel at Level_Offset */
HDWordReadSPI((Data_Addr + Level_Offset), &(Data_Values.Ch_2_Level_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Level_Sel < Level_Min)
    Data_Values.Ch_2_Level_Sel = Level_Min;
else if (Data_Values.Ch_2_Level_Sel > Level_Max)
    Data_Values.Ch_2_Level_Sel = Level_Max;

/* Ch_2_Waveform_Sel at Waveform_Offset*/
HDWordReadSPI((Data_Addr + Waveform_Offset), &(Data_Values.Ch_2_Waveform_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Waveform_Sel < Waveform_Min)
    Data_Values.Ch_2_Waveform_Sel = Waveform_Min;
else if (Data_Values.Ch_2_Waveform_Sel > Waveform_Max)
    Data_Values.Ch_2_Waveform_Sel = Waveform_Max;

/* Ch_2_Ratio_Sel at Ratio_Offset*/
HDWordReadSPI((Data_Addr + Ratio_Offset), &(Data_Values.Ch_2_Ratio_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Ratio_Sel < Ratio_Min)
    Data_Values.Ch_2_Ratio_Sel = Ratio_Min;
else if (Data_Values.Ch_2_Ratio_Sel > Ratio_Max)
    Data_Values.Ch_2_Ratio_Sel = Ratio_Max;

/* Ch_2_Sweep_Stop_Freq_Sel at Sweep_Stop_Freq_Offset*/
HDWordReadSPI((Data_Addr + Sweep_Stop_Freq_Offset), &(Data_Values.Ch_2_Sweep_Stop_Freq_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Sweep_Stop_Freq_Sel < Sweep_Stop_Min)
    Data_Values.Ch_2_Sweep_Stop_Freq_Sel = Sweep_Stop_Min;
else if (Data_Values.Ch_2_Sweep_Stop_Freq_Sel > Sweep_Stop_Max)
    Data_Values.Ch_2_Sweep_Stop_Freq_Sel = Sweep_Stop_Max;

/* Ch_2_Sweep_Type_Sel at Sweep_Type_Offset*/
HDWordReadSPI((Data_Addr + Sweep_Type_Offset), &(Data_Values.Ch_2_Sweep_Type_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Sweep_Type_Sel < Sweep_Type_Min)
    Data_Values.Ch_2_Sweep_Type_Sel = Sweep_Type_Min;
else if (Data_Values.Ch_2_Sweep_Type_Sel > Sweep_Type_Max)
    Data_Values.Ch_2_Sweep_Type_Sel = Sweep_Type_Max;

/* Ch_2_Sweep_Time_Sel at Sweep_Type_Offset*/
HDWordReadSPI((Data_Addr + Sweep_Time_Offset), &(Data_Values.Ch_2_Sweep_Time_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Sweep_Time_Sel < Sweep_Time_Min)
    Data_Values.Ch_2_Sweep_Time_Sel = Sweep_Time_Min;
else if (Data_Values.Ch_2_Sweep_Time_Sel > Sweep_Time_Max)
    Data_Values.Ch_2_Sweep_Time_Sel = Sweep_Time_Max;

/* Ch_2_Phase_Sel at Phase_Offset*/
HDWordReadSPI((Data_Addr + Phase_Offset), &(Data_Values.Ch_2_Phase_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Phase_Sel < Phase_Min)
    Data_Values.Ch_2_Phase_Sel = Phase_Min;
else if (Data_Values.Ch_2_Phase_Sel > Phase_Max)
    Data_Values.Ch_2_Phase_Sel = Phase_Max;

/* Ch_2_AM_Sel at AM_Offset*/
HDWordReadSPI((Data_Addr + AM_Offset), &(Data_Values.Ch_2_AM_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_AM_Sel < AM_On_Min)
    Data_Values.Ch_2_AM_Sel = AM_On_Min;
else if (Data_Values.Ch_2_AM_Sel > AM_On_Max)
    Data_Values.Ch_2_AM_Sel = AM_On_Max;

/* Ch_2_FM_Sel at FM_Offset*/
HDWordReadSPI((Data_Addr + FM_Offset), &(Data_Values.Ch_2_FM_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_FM_Sel < FM_On_Min)
    Data_Values.Ch_2_FM_Sel = FM_On_Min;
else if (Data_Values.Ch_2_FM_Sel > FM_On_Max)
    Data_Values.Ch_2_FM_Sel = FM_On_Max;

/* Ch_2_Pulse_Sel at Pulse_Offset*/
HDWordReadSPI((Data_Addr + Pulse_Offset), &(Data_Values.Ch_2_Pulse_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_Pulse_Sel < Pulse_On_Min)
    Data_Values.Ch_2_Pulse_Sel = Pulse_On_Min;
else if (Data_Values.Ch_2_Pulse_Sel > Pulse_On_Max)
    Data_Values.Ch_2_Pulse_Sel = Pulse_On_Max;

/* Ch_2_AM_Level_Sel at AM_Level_Offset*/
HDWordReadSPI((Data_Addr + AM_Level_Offset), &(Data_Values.Ch_2_AM_Level_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_AM_Level_Sel < AM_Depth_Min)
    Data_Values.Ch_2_AM_Level_Sel = AM_Depth_Min;
else if (Data_Values.Ch_2_AM_Level_Sel > AM_Depth_Max)
    Data_Values.Ch_2_AM_Level_Sel = AM_Depth_Max;

/* Ch_2_AM_Freq_Sel at AM_Freq_Offset*/
HDWordReadSPI((Data_Addr + AM_Freq_Offset), &(Data_Values.Ch_2_AM_Freq_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_AM_Freq_Sel < AM_Freq_Min)
    Data_Values.Ch_2_AM_Freq_Sel = AM_Freq_Min;
else if (Data_Values.Ch_2_AM_Freq_Sel > AM_Freq_Max)
    Data_Values.Ch_2_AM_Freq_Sel = AM_Freq_Max;

/* Ch_2_FM_Freq_Sel at FM_Freq_Offset*/
HDWordReadSPI((Data_Addr + FM_Freq_Offset), &(Data_Values.Ch_2_FM_Freq_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_FM_Freq_Sel < FM_Freq_Min)
    Data_Values.Ch_2_FM_Freq_Sel = FM_Freq_Min;
else if (Data_Values.Ch_2_FM_Freq_Sel > FM_Freq_Max)
    Data_Values.Ch_2_FM_Freq_Sel = FM_Freq_Max;

/* Ch_2_FM_Escursion_Sel at FM_Excursion_Offset*/
HDWordReadSPI((Data_Addr + FM_Excursion_Offset), &(Data_Values.Ch_2_FM_Excursion_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_FM_Excursion_Sel < FM_Excursion_Min)
    Data_Values.Ch_2_FM_Excursion_Sel = FM_Excursion_Min;
else if (Data_Values.Ch_2_FM_Excursion_Sel > FM_Excursion_Max)
    Data_Values.Ch_2_FM_Excursion_Sel = FM_Excursion_Max;

/* Ch_2_PM_Cycles_On_Sel at Cycles_On_Offset*/
HDWordReadSPI((Data_Addr + Cycles_On_Offset), &(Data_Values.Ch_2_PM_Cycles_On_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_PM_Cycles_On_Sel < PM_Cycles_On_Min)
    Data_Values.Ch_2_PM_Cycles_On_Sel = PM_Cycles_On_Min;
else if (Data_Values.Ch_2_PM_Cycles_On_Sel > PM_Cycles_On_Max)
    Data_Values.Ch_2_PM_Cycles_On_Sel = PM_Cycles_On_Max;

/* Ch_2_PM_Cycles_Off_Sel at Cycles_Of_Offset*/
HDWordReadSPI((Data_Addr + Cycles_Off_Offset), &(Data_Values.Ch_2_PM_Cycles_Off_Sel), 4 );
/* Check limits and fix if necessary*/
if (Data_Values.Ch_2_PM_Cycles_Off_Sel < PM_Cycles_Off_Min)
    Data_Values.Ch_2_PM_Cycles_Off_Sel = PM_Cycles_Off_Min;
else if (Data_Values.Ch_2_PM_Cycles_Off_Sel > PM_Cycles_Off_Max)
    Data_Values.Ch_2_PM_Cycles_Off_Sel = PM_Cycles_Off_Max;
  

/* deselect EEPROM*/
CselClearEEPROM;
}


/************************************************************************/
/* 	Write Line 1 and Line 2 of LCD					*/
/************************************************************************/
void Update_LCD(void)
{

/* Set Line 1 Address */
Write_Control_LCD_CSEL;
LCD_Data((LCD_Line_1));
DelayUs(LCD_Settle_Before_Write);
Write_Control_Enable_LCD_CSEL;
DelayUs(LCD_Delay_After_Write);
Write_Control_LCD_CSEL;

DelayMs(LCD_Settle_After_Control);

/*set the pointers at the line array adddresses */
line1_ptr = &(line1[0]);
for (loop = 0; loop < 16; loop++)
	{
		/* Write string 1 to LCD */
		Write_Data_Enable_LCD_CSEL;
		LCD_Data((char)*line1_ptr);
		DelayUs(LCD_Settle_Before_Write);
                Write_Data_LCD_CSEL;
		DelayUs(LCD_Delay_After_Write);
		Write_Data_Enable_LCD_CSEL;
		line1_ptr++;
	}


/* Set Line 2 Address */
Write_Control_LCD_CSEL;
LCD_Data((LCD_Line_2));
DelayUs(LCD_Settle_Before_Write);
Write_Control_Enable_LCD_CSEL;
DelayUs(LCD_Delay_After_Write);
Write_Control_LCD_CSEL;
DelayMs(LCD_Settle_After_Control);

/*set the pointers at the line array adddresses */
line2_ptr = &(line2[0]);
for (loop = 0; loop < 16; loop++)
	{
		/* Write string 2 to LCD */
		Write_Data_Enable_LCD_CSEL;
		LCD_Data((char)*line2_ptr);
         	DelayUs(LCD_Settle_Before_Write);
		Write_Data_LCD_CSEL;
		DelayUs(LCD_Delay_After_Write);
		Write_Data_Enable_LCD_CSEL;
		line2_ptr++;
	}	
	
}


/************************************************************************/
/* 						Init  			*/
/*  This is for a standard 16 x 2 character LCD... 			*/
/************************************************************************/
void Init_LCD(void)
{
/* Function Select */
Write_Control_LCD_CSEL;
LCD_Data(LCD_Function);
Write_Control_Enable_LCD_CSEL;
DelayUs(LCD_Settle_Before_Write);
Write_Control_LCD_CSEL;

DelayUs(LCD_Delay_After_Write);

/* Display On */
LCD_Data(LCD_Display_On);
Write_Control_Enable_LCD_CSEL;
DelayUs(LCD_Settle_Before_Write);
Write_Control_LCD_CSEL;

DelayUs(LCD_Delay_After_Write);

/* Display Clear */
LCD_Data(LCD_Display_Clear);
Write_Control_Enable_LCD_CSEL;
DelayUs(LCD_Settle_Before_Write);
Write_Control_LCD_CSEL;

DelayMs(LCD_Settle_After_Control);

/* Entry Mode */
LCD_Data(LCD_Display_Entry_Mode);
Write_Control_Enable_LCD_CSEL;
DelayUs(LCD_Settle_Before_Write);
Write_Control_LCD_CSEL;
DelayUs(LCD_Delay_After_Write);

/* Display Clear */
Write_Control_LCD_CSEL;
LCD_Data(LCD_Display_Clear);
Write_Control_Enable_LCD_CSEL;
DelayUs(LCD_Delay_After_Write);
Write_Control_LCD_CSEL;

DelayMs(LCD_Settle_After_Control);
}




/************************************************************************/
/* Display relevant data on the LCD 			
Need to build display lies for:
Display_Fn = 
	Choosing_Fn	
	Volume_Mode
	XO_Mode 
	Save_Mode
	Load_Mode
	EQ_Mode
	Change_Mode							*/
/*
and the following explicit functions
	XO_Band_Sel_Fn
	XO_Parm_Sel
	XO_LF_Parm_Fn
	XO_HF_Parm_Fn
	XO_SL_Parm_Fn
	XO_SU_Parm_Fn
	XO_Del_Parm_Fn
	XO_Att_Parm_Fn

	Save_Sel_Fn

	Load_Sel_Fn							*/

/*
#define EQ_Sel_Fn 				
#define EQ_Freq_Sel_Fn 			
#define EQ_Q_Sel_Fn  	
#define EQ_Gain_Sel_Fn 	
#define EQ_Type_Sel_Fn 	
									*/
/************************************************************************/
void Display_Data(int Display_Fn, int Display_Value)
{
char loop;
int Display_Value_Temp;
int Display_Ptr_Offset;  /* used to reference lookup into display string*/

/* Use this to remember if the Display Value needs a - sign */
Display_Value_Temp = Display_Value;

/*set the pointers at the line array adddresses */
/* The idea here is to use pointers into the Line1 and Line 2 arrays */
/* The code efficienct using direct array operations is abysmal.     */
/* Remember that this fuction builds the lines from right to left    */
/* You did the because it makes it easer to build the numbers....    */
/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;

/* Work out if we are displayng to level displays */
        if((Display_Fn >= Ch_1_Freq) && (Display_Fn <= Ch_2_Pulse))
        {
            if (Display_Fn <= Ch_1_Pulse)
                Display_Ptr_Offset = Ch_1_Freq;
            else
                Display_Ptr_Offset = Ch_2_Freq;
        }
        else
            Display_Ptr_Offset = 0;


/********************************************************************/
/* 					Build Line 1 string in here */
/********************************************************************/

/************************************************************************/
/* Choosing Value - user selecting the fuinction to modify              */
/************************************************************************/
	if (Display_Fn == Choosing_Fn)
		{
		*line1_ptr = Choose_Str[loop];
		*line2_ptr = Choice_Display[Display_Value - 1][loop];					
		}


/************************************************************************/
/* Channel 1 mode Select 					 	*/
/************************************************************************/
        else if (Display_Fn == Ch_1_Mode)
		{
		/* Build string */
		*line1_ptr = Line1_Channel_1_Init[loop];
                *line2_ptr = Channel_Setup_Choice_Display[Display_Value - Ch_1_Freq][loop];
		}

/************************************************************************/
/* Channel 2 mode Select 					 	*/
/************************************************************************/
        else if (Display_Fn == Ch_2_Mode)
		{
		/* Build string */
		*line1_ptr = Line1_Channel_2_Init[loop];
                *line2_ptr = Channel_Setup_Choice_Display[Display_Value - Ch_2_Freq][loop];
		}

/************************************************************************/
/* Save Function display - for Normal Save functionality        	*/
/************************************************************************/
        else if ((Display_Fn == Save_Sel_Fn) && (Display_Value <= Max_Mem_Banks))
		{
		/* Build string */
		*line1_ptr = Save_Str[loop];

		/* Convert 0-3 integer to 1-3 ASCII for bank number */
		if (loop == 13)
			*line1_ptr = Display_Value + 49;
		}
/************************************************************************/
/* Save Function display - for Erasing EEPROM                   	*/
/************************************************************************/
        else if ((Display_Fn == Save_Sel_Fn) && (Display_Value > Max_Mem_Banks))
		{
		/* Build string */
		*line1_ptr = Erase_Str1[loop];
                if (Display_Value == Mem_Banks_Do_Not_Erase)
                    *line2_ptr = Erase_Str2_No[loop];
                else if (Display_Value == Mem_Banks_Erase)
                    *line2_ptr = Erase_Str2_Yes[loop];

		/* Convert display Vaue into "Yes or No" */
		if ((loop == 14) && (Display_Value == Mem_Banks_Do_Not_Erase))
			*line1_ptr = 'N';
                else if ((loop == 14) && (Display_Value == Mem_Banks_Erase))
			*line1_ptr = 'Y';
		}


/************************************************************************/
/*Load function display							*/
/************************************************************************/
	else if (Display_Fn == Load_Sel_Fn)
		{
		/* Build string */
		*line1_ptr = Load_Str[loop];

		/* Convert 0-3 integer to 1-3 ASCII for bank number */
		if (loop == 15)
			*line1_ptr = Display_Value + 49;
		}


/************************************************************************/
/* Ch_1 Freequency Selection Function display                           */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Freq) || (Display_Fn == Ch_2_Freq))
		{
		/* Build string */
		*line1_ptr = Channel_Setup_Choice_Display[Display_Fn - Display_Ptr_Offset][loop];
                *line2_ptr = Setup_Choice_Units_Display[Display_Fn - Display_Ptr_Offset][loop];

          	if (loop == 5)
		{
                    *line2_ptr = 46;  /* 46 is "." */
		}

                /* Put up Frequency */
                else if (loop <= 6)
                    {
                        display_temp = Display_Value % 10;
			if (Display_Value > 0)
			    *line2_ptr = (display_temp)+48;
			else
			    *line2_ptr = Blank_Char;
			Display_Value = Display_Value/10;
                    }
		}



/************************************************************************/
/* Ch_1 Freequency Speed Select display                                 */
/************************************************************************/
        else if ((Display_Fn == Freq_Speed_Counter_1_fn) || (Display_Fn == Freq_Speed_Counter_2_fn))
		{
		/* Build string */
		*line1_ptr = Freq_Stepsize_Select_Display[0][loop];
                *line2_ptr = Freq_Stepsize_Select_Display[1][loop];

          	if (loop == 3)
		{
                    *line2_ptr = 46;  /* 46 is "." */
		}

                /* Put up Frequency */
                else if (loop <= 4)
                    {
                        display_temp = Display_Value % 10;
			if (Display_Value > 0)
			    *line2_ptr = (display_temp)+48;
			else
			    *line2_ptr = Blank_Char;
			Display_Value = Display_Value/10;
                    }
		}
        
/************************************************************************/
/* Ch_1 Level Selection Function display                                */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Level) || (Display_Fn == Ch_2_Level))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = Channel_Setup_Choice_Display[Display_Fn - Display_Ptr_Offset][loop];
                *line2_ptr = Setup_Choice_Units_Display[Display_Fn - Display_Ptr_Offset][loop];

                /* Put up level */
			if (loop == 4)
				{
				display_temp = Display_Value % 2;
				/* is there a 0.5dB step? */
				if (display_temp == 1)
					{
					*line2_ptr = (5)+48;  /* 48 is offset of "0" ASCII */
					}
				else
					{
					*line2_ptr = (0)+48;  /* 48 is offset of "0" ASCII */
					}
				Display_Value = Display_Value/2;
				}
			if (loop == 3)
				{
					*line2_ptr = 46;  /* 46 is "." */
				}
			if (loop < 3)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && (loop == 2))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}


/************************************************************************/
/* Ch_1 Waveform Selection Function display                             */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Waveform) || (Display_Fn == Ch_2_Waveform) )
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = Channel_Setup_Choice_Display[Display_Fn - Display_Ptr_Offset][loop];
                *line2_ptr = Waveform_Select_Display[Display_Value - Sine_Sel][loop];
		}
/************************************************************************/
/* Ch_1 Sweep, AM, FM and Pulse Selection Function display              */
/************************************************************************/

        else if ((Display_Fn == Ch_1_AM) || (Display_Fn == Ch_1_FM) || (Display_Fn == Ch_1_Pulse) ||
                  (Display_Fn == Ch_2_AM) || (Display_Fn == Ch_2_FM) || (Display_Fn == Ch_2_Pulse))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
                *line1_ptr = Channel_Setup_Choice_Display[(int)(Display_Fn - Display_Ptr_Offset)][loop];
                if (loop < 3)
                    {
                        if ((Display_Value == Sweep_Off) || (Display_Value == AM_Off) || (Display_Value == FM_Off) || (Display_Value == Pulse_Off))
                            *line2_ptr = Yes_No[1][loop];
                        else
                            *line2_ptr = Yes_No[0][loop];
                    }
		}

/************************************************************************/
/* Ch_1 Ch_1_PM_Cycles_On Selection Function display                                */
/************************************************************************/
        else if ((Display_Fn == Ch_1_PM_Cycles_On) || (Display_Fn == Ch_2_PM_Cycles_On))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = Setup_Cycles_On_Display[0][loop];
                *line2_ptr = Setup_Cycles_On_Display[1][loop];

                /* Put up number of cycles */
                        if (loop <= 3)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && (loop == 4))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}

/************************************************************************/
/* Ch_1 Ch_1_PM_Cycles_Off Selection Function display                                */
/************************************************************************/
        else if ((Display_Fn == Ch_1_PM_Cycles_Off) || (Display_Fn == Ch_2_PM_Cycles_Off))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = Setup_Cycles_Off_Display[0][loop];
                *line2_ptr = Setup_Cycles_Off_Display[1][loop];

                /* Put up number of cycles */
                        if (loop <= 3)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && (loop == 4))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}


/************************************************************************/
/* Ch_1 Phase Selection Function display                                */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Phase) || (Display_Fn == Ch_2_Phase))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = Channel_Setup_Choice_Display[Display_Fn - Display_Ptr_Offset][loop];
                *line2_ptr = Setup_Choice_Units_Display[Display_Fn - Display_Ptr_Offset][loop];

                /* mask off all but relevant bits*/
                if (display_temp > Phase_Max)
                    display_temp = Phase_Max;
                if (display_temp < Phase_Min)
                    display_temp = Phase_Min;

                /* Put up level */
			if (loop == 3)
				{
				*line2_ptr = 46;  /* 46 is "." */
				}
                        else if (loop <= 4)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && ((loop == 2)||(loop == 4)))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}
/************************************************************************/
/* Ch_1 Ratio Selection Function display                                */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Ratio) || (Display_Fn == Ch_2_Ratio))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = Ratio_Select_Display[0][loop];
                *line2_ptr = Ratio_Select_Display[1][loop];

                /* Put up level */
                        if (loop < 3)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && ((loop == 2)||(loop == 4)))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}
/************************************************************************/
/* Ch_1 Sweep Selection Function display                             */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Sweep) || (Display_Fn == Ch_2_Sweep) )
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = Channel_Setup_Choice_Display[3][loop];
                *line2_ptr = Sweep_Select_Display[Display_Value - Sweep_Off][loop];
		}

/************************************************************************/
/* Ch_1 Sweep Frequency Selection Function display                      */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Sweep_Stop) || (Display_Fn == Ch_2_Sweep_Stop))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = Sweep_Freq_Select_Display[0][loop];
                *line2_ptr = Sweep_Freq_Select_Display[1][loop];

                /* Put up level */
                        if (loop < 5)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && ((loop == 2)||(loop == 5)))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}

/************************************************************************/
/* Ch_1 Sweep Time Selection Function display                      */
/************************************************************************/
        else if ((Display_Fn == Ch_1_Sweep_Time) || (Display_Fn == Ch_2_Sweep_Time))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = Sweep_Freq_Time_Display[0][loop];
                *line2_ptr = Sweep_Freq_Time_Display[1][loop];

                /* Put up level */
                            if (loop == 2)
                                {
                                    *line2_ptr = 46;  	/* . */
                                }
                            else if (loop < 4)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && (loop == 1))
					{
					*line2_ptr = 48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}

/************************************************************************/
/* Ch_1 AM Level Selection Function display                                */
/************************************************************************/
        else if ((Display_Fn == Ch_1_AM_Level) || (Display_Fn == Ch_2_AM_Level))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = AM_Mod_Select_Display[0][loop];
                *line2_ptr = AM_Mod_Select_Display[1][loop];

                /* Put up level */
                        if (loop < 3)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && ((loop == 2)||(loop == 4)))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}
/************************************************************************/
/* Ch_1 AM Frequency Selection Function display                         */
/************************************************************************/
        else if ((Display_Fn == Ch_1_AM_Freq) || (Display_Fn == Ch_2_AM_Freq))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = AM_Freq_Select_Display[0][loop];
                *line2_ptr = AM_Freq_Select_Display[1][loop];

                /* Put up level */
                        if (loop < 4)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && ((loop == 2)||(loop == 4)))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}

/************************************************************************/
/* Ch_1 AM Level Selection Function display                                */
/************************************************************************/
        else if ((Display_Fn == Ch_1_FM_Excursion) || (Display_Fn == Ch_2_FM_Excursion))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = FM_Mod_Select_Display[0][loop];
                *line2_ptr = FM_Mod_Select_Display[1][loop];

                /* Put up level */
                        if (loop < 4)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && ((loop == 2)||(loop == 4)))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}
/************************************************************************/
/* Ch_1 AM Frequency Selection Function display                         */
/************************************************************************/
        else if ((Display_Fn == Ch_1_FM_Freq) || (Display_Fn == Ch_2_FM_Freq))
		{
		/* Build string - this should automatically put up the  */
                /* right string by referencing off Ch_1_Freq            */
		*line1_ptr = FM_Freq_Select_Display[0][loop];
                *line2_ptr = FM_Freq_Select_Display[1][loop];

                /* Put up level */
                        if (loop < 4)
				{
				display_temp = Display_Value % 10;
				/* DISPLAY ZERO on leading character only if it actually is ZERO! */
				if((Display_Value == 0) && ((loop == 2)||(loop == 4)))
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					}
				else if (Display_Value > 0)
					{
					*line2_ptr = (display_temp)+48;  	/* 48 is offset of "0" ASCII */
					Display_Value = Display_Value/10;
					}
				}
		}


	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 							*/
	/************************************************************************/
Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}


//Print_Data(int Number_of_Words, char *ptr_to_Data)
void Print_Data(int Number_of_Words, unsigned char *ptr_to_Data, unsigned int Address_to_Print)
{
char loop;
unsigned int display_temp, display_data;
unsigned int Address_Temp, Address_Temp_Hex;

	/*set the pointers at the line array adddresses */
	/* The idea here is to use pointers into the Line1 and Line 2 arrays */
	/* The code efficiency using direct array operations is abysmal.     */
	/* Remember that this fuction builds the lines from right to left    */
	/* You did the because it makes it easer to build the numbers....    */
	/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
display_data = *ptr_to_Data;    /* initial dtaa to diaplay */
Address_Temp = Address_to_Print;
Address_Temp_Hex = Address_to_Print;

for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;

	display_temp = display_data % 16;
	if (loop > (15 - 2*Number_of_Words))
		{
			if (display_temp <= (unsigned char)9)
				*line1_ptr = (display_temp)+48;
			else
				*line1_ptr = (display_temp)+55;
		}
	display_data = display_data>>4;
	if (!(loop % 2)  && (((15 - loop)/2) < Number_of_Words))  /* then need to read next byte */
		{
		ptr_to_Data++;
		display_data = *ptr_to_Data;
		}

	display_temp = Address_Temp % 10;
	if ((Address_Temp > (unsigned int)0) || (loop > 14))
		*line2_ptr = (display_temp)+48;
	else
		*line2_ptr = Blank_Char;
	Address_Temp = Address_Temp/10;


	if ((loop <= 8) && (loop > 4))
		{
			display_temp = Address_Temp_Hex % 16;
			if (display_temp <= (unsigned char)9)
				*line2_ptr = (display_temp)+48;
			else
				*line2_ptr = (display_temp)+55;
			Address_Temp_Hex = Address_Temp_Hex>>4;
		}


	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 							*/
	/************************************************************************/

Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}


void Display_Debug_Data(unsigned int Display_Value)
{
char loop;
unsigned int Display_Value_Temp;
		/* Use this to remember if the Display Value needs a - sign */
Display_Value_Temp = Display_Value;
		/*set the pointers at the line array adddresses */
		/* The idea here is to use pointers into the Line1 and Line 2 arrays */
		/* The code efficienct using direct array operations is abysmal.     */
		/* Remember that this fuction builds the lines from right to left    */
		/* You did the because it makes it easer to build the numbers....    */
		/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;

	display_temp = Display_Value_Temp % 16;
	if ((Display_Value_Temp != 0) || (loop > 11))
		{
			if (display_temp <= 9)
				*line1_ptr = ((unsigned char)display_temp)+48;
			else
				*line1_ptr = ((unsigned char)display_temp)+55;
		}
	else
		*line1_ptr = Blank_Char;
	Display_Value_Temp = Display_Value_Temp/16;


	display_temp = Display_Value % 10;
	if ((Display_Value != 0) || (loop < 11))
		*line2_ptr = ((unsigned char)display_temp)+48;
	else
		*line2_ptr = Blank_Char;
	Display_Value = Display_Value/10;


	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 							*/
	/************************************************************************/

Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}


void Display_Debug_Data_Char(unsigned char Display_Value)
{
char loop;
int Display_Value_Temp;

/* Use this to remember if the Display Value needs a - sign */
Display_Value_Temp = Display_Value;

/*set the pointers at the line array adddresses */
/* The idea here is to use pointers into the Line1 and Line 2 arrays */
/* The code efficienct using direct array operations is abysmal.     */
/* Remember that this fuction builds the lines from right to left    */
/* You did the because it makes it easer to build the numbers....    */
/* The value of the strings is contained in *line1_ptr and *line2_ptr*/
line1_ptr = &(line1[15]);
line2_ptr = &(line2[15]);
for (loop = 15; loop >= 0; loop--)
	{
	/* initialise the strings */
	*line1_ptr = Blank_Char;
	*line2_ptr = Blank_Char;

	display_temp = (unsigned char) Display_Value_Temp % 16;
	if ((Display_Value_Temp > 0) || (loop > 13))
		{
			if (display_temp <= 9)
				*line1_ptr = (unsigned char) (display_temp)+48;
			else
				*line1_ptr = (unsigned char) (display_temp)+55;
		}
	else
		*line1_ptr = Blank_Char;
	Display_Value_Temp = (unsigned char) Display_Value_Temp/16;


	display_temp = (unsigned char)Display_Value % 10;
	if ((Display_Value > (unsigned char)0) || (loop > (char)13))
		*line2_ptr = (unsigned char) (display_temp)+48;
	else
		*line2_ptr = Blank_Char;
	Display_Value = (unsigned char) Display_Value/10;


	/* Now decrement the pointers Line 1 and Line 2  */
	line1_ptr--;
	line2_ptr--;
	}
	/************************************************************************/
	/* END of Loop to 16 							*/
	/************************************************************************/
Update_LCD();		/* Writes Line1 and Line2 Strings to LCD */
}




/* Call this to update all Atten settings etc 					*/
/*	inputs									*/
/*		DAC_Atten - attenution sought in 0.5dB steps			*/
/*		Data_Values (static variable) with band attenuation in 0.5dB steps*/
/*	Outputs									*/
/*		None								*/
/*	Calls Write Atten Loop with required attentaion in o.5dB steps		*/
void Update_Atten_Set()
{
int temp_atten1;
int temp_atten2;

/* do channel 1 first */
temp_atten1 = (unsigned int)Level_Max - (unsigned int) Data_Values.Ch_1_Level_Sel;
	if (temp_atten1 <  Level_Min)
		temp_atten1 = (unsigned int)Level_Min;
	if (temp_atten1 > Level_Max)
		temp_atten1 = (unsigned int)Level_Max;

/* then channel 2*/
temp_atten2 = (unsigned int)Level_Max - (unsigned int) Data_Values.Ch_2_Level_Sel;
	if (temp_atten2 <  Level_Min)
		temp_atten2 = (unsigned int)Level_Min;
	if (temp_atten2 > Level_Max)
		temp_atten2 = (unsigned int)Level_Max;

/* now write attenuation for the PGA2320 devices  */
Write_PGA2320((unsigned char)temp_atten1, (unsigned char)temp_atten2);
}

/****************************************************************************/	
/* Uses the following globals                                               */
/* s_count                                                                  */
/* Old_Rot_Status    Records state of rotary control                        */
/* Fast_Loop_Counter  Used to set Fast_Cnt                                  */
/* Slow_Loop_Counter  Used to see if user turning control continuously      */
/*                                                                          */
/*                                                                          */
/* This sets is the ISR that deals with key presses etc                     */
/****************************************************************************/	
void __ISR(_TIMER_1_VECTOR, ipl4) Timer1ISR(void)
{
    unsigned char Current_Rot_Status = 0;

    mPORTBToggleBits(BIT_9);
    Nop();
        // disable interrupts from T1
        // but do not disable interrupts from the SPI interface
        // intStatus = INTDisableInterrupts();
//        INTEnable(INT_T1, INT_DISABLED);

        /*Read Rot Status and mask off bits of interest */
	Current_Rot_Status = HMI_Port & (Vals_0 + Vals_1 + Exit + Sel);  
	Key_Just_Pressed = (Current_Rot_Status != Old_Rot_Status);

	/* Only do this if something has changed - but neither of the select or exit buttons are pressed */
	if (Key_Just_Pressed  && !(Current_Rot_Status & (Sel + Exit)))
	{
            /* Old Type Encoder is biphase input on two ports with */
            /*      Each detent click of the resolver being one change in phase*/
            /*      Thus one gray code change of the inputs occurs per click*/
            if Encoder_Type_Is_Old
            {
		if (Old_Rot_Status == Phase_0)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_1)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;
		}
		else if (Old_Rot_Status == Phase_1)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_2)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;			
		}
		else if (Old_Rot_Status == Phase_2)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_3)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;
		}
		else if (Old_Rot_Status == Phase_3)  /* From here need to work out if going up or down */
		{
			if (Current_Rot_Status == Phase_0)
				Keypress_Read = Rot_Up;
			else
				Keypress_Read = Rot_Down;			
		}
            }
            /* Type A Encoder is biphase input on two ports with */
            /*      Each detent click of the resolver being four changes in phase*/
            /*      Thus one complete cycle of gray code occurs per click*/
            /*      Type A goes 11 -> 10 -> 00 -> 01 -> 11 and vice versa*/
            else if Encoder_Type_Is_TypeA
            {
                /* Phase_0 = 00*/
                /* From here need to work out if going up or down or reading crap*/
                /* CW 00 -> 01 */
                /* CCW 00-> 10 */
                /* But we do not want to act on these as it is mid-click*/
                if (Old_Rot_Status == Phase_0)
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
		}
                /* From here need to work out if going up or down or reading crap*/
                /* CW 01 -> 11 */
                /* CCW 01-> 00 */
                /* But we do not want to act on these as it is mid-click*/
		else if (Old_Rot_Status == Phase_1) 
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
		}
                /* From here need to work out if going up or down or reading crap*/
                /* CW 11 -> 10 */
                /* CCW 11-> 01 */
                /* This is it... do something*/
		else if (Old_Rot_Status == Phase_2)  
		{
			if (Current_Rot_Status == Phase_3)
				Keypress_Read = Rot_Down;
			else if (Current_Rot_Status == Phase_1)
				Keypress_Read = Rot_Up;
                        else Keypress_Read = 0;
		}
                /* From here need to work out if going up or down or reading crap*/
                /* CW 10 -> 00 */
                /* CCW 10-> 11 */
                /* But we do not want to act on these as it is mid-click*/
		else if (Old_Rot_Status == Phase_3) 
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
                }
            }
            /* Type B Encoder is biphase input on two ports with */
            /*      Each detent click of the resolver being three changes in phase*/
            /*      Thus one complete cycle of pseudo gray code occurs per click*/
            /*      Type B is asymetric for CW and CCW rotation*/
            /*      Type B goes 11 -> 00 -> 01 -> 11 for CW*/
            /*      Type B goes 00 -> 01 -> 00 -> 11 for CCW*/
            else if Encoder_Type_Is_TypeB
            {
                /* Phase_0 = 00*/
                /* From here need to work out if going up or down or reading crap*/
                /* CW 00 -> 01 */
                /* CCW 00-> 11 */
                /* But we do not want to act on these */
		if (Old_Rot_Status == Phase_0)
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
		}
                /* Phase_1 = 01*/
                /* From here need to work out if going up or down or reading crap*/
                /* CW  01 -> 11 */
                /* CCW 01 -> 00 */
                /* But we do not want to act on these */
		else if (Old_Rot_Status == Phase_1)
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
		}
                /* Phase_2 = 11*/
                /* OK This is it... */
                /* CW  11 -> 00 */
                /* CCW 11 -> 01 */
                /* But we do not want to act on these */
		else if (Old_Rot_Status == Phase_2)
		{
			if (Current_Rot_Status == Phase_0)
                            Keypress_Read = Rot_Up;
			else if (Current_Rot_Status == Phase_3)
                            Keypress_Read = Rot_Down;
                        else
                            Keypress_Read = 0;
		}
                /* Phase_3 = 10*/
                /* We are reading crap - illegal for this encoder*/
                /* We do not want to act on these */
		else if (Old_Rot_Status == Phase_3)  /* From here need to work out if going up or down */
		{
                   /* Have read crap*/
                    Keypress_Read = 0;
                }
                else
                {
                    Keypress_Read = 0;
                }
            }
	
	Slow_Loop_Counter = Slow_Counter_Init;	/* If key just pressed initialise slow counter */
	
	if (Fast_Loop_Counter > (unsigned int)0)
		Fast_Loop_Counter--;
	}
	/* else if key pressed is not a rotation, then...*/
        else if (Key_Just_Pressed && (Current_Rot_Status & (Sel + Exit)))
	{
		if (Current_Rot_Status & Sel)
			Keypress_Read = Sel;			
		if (Current_Rot_Status & Exit)
			Keypress_Read = Exit;			
		Fast_Change = 1;
	}		
	else    /* else not Key_just_Pressed */
	{
		Keypress_Read = 0;
		if (Slow_Loop_Counter > (unsigned int)0)
			Slow_Loop_Counter--;
	}

	if (Slow_Loop_Counter == (unsigned int)0)
	{
		Fast_Change = 1;
		Fast_Loop_Counter = Fast_Counter_Init;
	}

	if (Fast_Loop_Counter == (unsigned int)0)   /* move into fast change of value for user */
	{
                Fast_Change = 100;
	}
			
	Old_Rot_Status = Current_Rot_Status;   /*Record Rot Status to allow comparison next ISR loop */

mPORTBToggleBits(BIT_9);
Nop();
// set timer to zero then clear the interrupt flag
        TMR1 = 0x0000;
        mT1ClearIntFlag();
}

/****************************************************************************/
/* Uses the following globals                                               */
/*                                                                          */
/*                                                                          */
/*                                                                          */
// disable interrupts
//intStatus = INTDisableSystemMultiVectoredInt();
// do some critical code here
// restore interrupts to the previous state
//INTRestoreInterrupts(intStatus);
/*                                                                          */
/* This sets is the ISR that deals with the I2S interface                   */
/* It ultimately needs to generate DAC data at the sample rate              */
/****************************************************************************/
void __ISR(_SPI_2_VECTOR, ipl3SRS) SPI2InterruptHandler(void) {

    /* test Code */
mPORTBToggleBits(BIT_8);
//Nop();
mPORTBToggleBits(BIT_8);
//Nop();

/* calculate channel 1 sweep stuff */
if(Ch1_Sweep_On == 1)
{
    Ch1_Sweep_Phase_Reg += Ch1_Sweep_Phase_Inc;
    if (Ch1_Sweep_Upwards)
    {
        if (Ch1_Sweep_Phase_Reg > Ch1_Sweep_End)
            Ch1_Sweep_Phase_Reg = 0;
    }
    else 
    {
        if (Ch1_Sweep_Phase_Reg < Ch1_Sweep_End)
            Ch1_Sweep_Phase_Reg = 0;
    }
}
/* calculate channel 2 sweep stuff */
if(Ch2_Sweep_On == 1)
{
    Ch2_Sweep_Phase_Reg += Ch2_Sweep_Phase_Inc;
    if (Ch2_Sweep_Upwards == 1)
    {
        if (Ch2_Sweep_Phase_Reg > Ch2_Sweep_End)
            Ch2_Sweep_Phase_Reg = 0;
    }
    else
    {
        if (Ch2_Sweep_Phase_Reg < Ch2_Sweep_End)
            Ch2_Sweep_Phase_Reg = 0;
    }
}

/* implement AM channel 1*/
/* Run the Ch1 DDS */
Ch1_AM_Phase_Reg += Ch1_AM_Phase_Inc;
if (Ch1_AM_On)
{
    Ch1_AM_Temp = (long long)Sine_LUT[Ch1_AM_Phase_Reg >> 20];  /* this is +/-23 bits*/
    Ch1_AM_Temp += 0x7fffff;        /* this should make it from 0-ffffff*/
    Ch1_AM_Temp *= (long long) Ch1_AM_Mult;
    Ch1_AM_Temp =  Ch1_AM_Temp >>31;   /* this scales to +/- AM level*/
    Ch1_AM_Temp = 0xffffff - Ch1_AM_Temp;       /* shifts to oxffffff times AM*/
}
else
{
    Ch1_AM_Temp = 0xffffff;
}

/* implement AM channel 2*/
Ch2_AM_Phase_Reg += Ch2_AM_Phase_Inc;
if (Ch2_AM_On)
{
    Ch2_AM_Temp = (long long)Sine_LUT[Ch2_AM_Phase_Reg >> 20];  /* this is +/-23 bits*/
    Ch2_AM_Temp += 0x7fffff;        /* this should make it from 0-ffffff*/
    Ch2_AM_Temp *= (long long)Ch2_AM_Mult;
    Ch2_AM_Temp =  Ch2_AM_Temp >> 31;   /* this scales to +/- AM level*/
    Ch2_AM_Temp = 0xffffff - Ch2_AM_Temp;       /* shifts to oxffffff times AM*/
}
else
{
    Ch2_AM_Temp = 0xffffff;
}

/* implement FM channel 1*/
/* Run the Ch1 DDS */
/* the FM Phase offset gets added to the phase increment for the channel*/
Ch1_FM_Phase_Reg += Ch1_FM_Phase_Inc;
if (Ch1_FM_On)
{
    Ch1_FM_Phase_Offset = Ch1_FM_LUT[Ch1_FM_Phase_Reg >> 21];
}
else
    Ch1_FM_Phase_Offset = 0;

/* implement FM channel 2*/
/* the FM Phase offset gets added to the phase increment for the channel*/
Ch2_FM_Phase_Reg += Ch2_FM_Phase_Inc;
if (Ch2_FM_On)
{
    Ch2_FM_Phase_Offset = Ch2_FM_LUT[Ch2_FM_Phase_Reg >> 21];
}
else
    Ch2_FM_Phase_Offset = 0;

/* Run main DDS accumulators channe 1*/
Phase_Acc_Ch1 += Phase_Inc_Ch1;
Phase_Acc_Ch1 += Ch1_FM_Phase_Offset;
if(Ch1_Sweep_On == 1)
    Phase_Reg_Temp = (int)(Ch1_Sweep_Phase_Reg >>10);
Phase_Acc_Ch1 += Phase_Reg_Temp;
/* Run main DDS accumulators channe 1*/
Phase_Acc_Ch2 += Phase_Inc_Ch2;
Phase_Acc_Ch2 += Ch2_FM_Phase_Offset;
if(Ch2_Sweep_On == 1)
    Phase_Reg_Temp = (int)(Ch2_Sweep_Phase_Reg >>10);
Phase_Acc_Ch2 += Phase_Reg_Temp;

/* implement Pulse modulation */
if (Old_Phase_Acc_Ch1 > Phase_Acc_Ch1) /* then we have wrapped around zero*/
{
    if(Ch1_Pulse_Active == Pulse_On) /* pulse mod is on*/
    {
        if(Ch1_Pulse_Output_Active) /* and the output is on*/
        {
            Ch1_Pulse_Counter--; /* decrement pulse counter*/
            if (!Ch1_Pulse_Counter)  /*if all pulses have been output*/
            {
                Ch1_Pulse_Output_Active = 0; /* turn output off */
                Ch1_Pulse_Counter = Ch1_Pulse_Cycles_InActive; /* then count this many pulses for "off"*/
            }
        }
        else /* the output is therefore off*/
        {
            Ch1_Pulse_Counter--; /* decrement pulse counter*/
            if (!Ch1_Pulse_Counter)  /*if all pulses in "off period" have been output*/
            {
                Ch1_Pulse_Output_Active = 1; /* turn output on */
                Ch1_Pulse_Counter = Ch1_Pulse_Cycles_Active; /* then count this many pulses for "on"*/
            }
        }
    }
    else
    {
        Ch1_Pulse_Output_Active = 1;
    }
}
/* and channel 2 pulse mod*/
if (Old_Phase_Acc_Ch2 > Phase_Acc_Ch2) /* then we have wrapped around zero*/
{
    if(Ch2_Pulse_Active == Pulse_On) /* pulse mod is on*/
    {
        if(Ch2_Pulse_Output_Active) /* and the output is on*/
        {
            Ch2_Pulse_Counter--; /* decrement pulse counter*/
            if (!Ch2_Pulse_Counter)  /*if all pulses have been output*/
            {
                Ch2_Pulse_Output_Active = 0; /* turn output off */
                Ch2_Pulse_Counter = Ch2_Pulse_Cycles_InActive; /* then count this many pulses for "off"*/
            }
        }
        else /* the output is therefore off*/
        {
            Ch2_Pulse_Counter--; /* decrement pulse counter*/
            if (!Ch2_Pulse_Counter)  /*if all pulses in "off period" have been output*/
            {
                Ch2_Pulse_Output_Active = 1; /* turn output on */
                Ch2_Pulse_Counter = Ch2_Pulse_Cycles_Active; /* then count this many pulses for "on"*/
            }
        }
    }
    else
    {
        Ch2_Pulse_Output_Active = 1;
    }
}


if (Old_Phase_Acc_Ch2 > Phase_Acc_Ch2)
{
    Cycles_On_Ch2++;
}

/* right shift to address LUT */
Delta_Phase_Acc_Ch1 = (Phase_Acc_Ch1  + Ch1_Phase_Delta);
Phase_Reg_Temp = Delta_Phase_Acc_Ch1 >> 20;
Delta_Phase_Acc_Ch1 = Delta_Phase_Acc_Ch1 & 0xfffff;
Next_Phase_Reg_Temp = Phase_Reg_Temp + 1;
if (Next_Phase_Reg_Temp == 4096)
    Next_Phase_Reg_Temp = 0;
/*delta Volts in step */
Interpolated_Output = (long int)Ch1_LUT[Phase_Reg_Temp];
Temp_Interpolated_Output = (Ch1_LUT[Next_Phase_Reg_Temp] - Ch1_LUT[Phase_Reg_Temp]);
Temp_Interpolated_Output = Temp_Interpolated_Output * Delta_Phase_Acc_Ch1;
Temp_Interpolated_Output = Temp_Interpolated_Output /0x100000;
Interpolated_Output = Interpolated_Output + Temp_Interpolated_Output;
//Interpolated_Output = Temp_Interpolated_Output <<9;
/* implement amplitude scaling */
if(Ch1_Level_Mult != 0xff)
{
    Interpolated_Output *= (long int)Ch1_Level_Mult;
    Interpolated_Output = Interpolated_Output >>8;
}
Interpolated_Output = Interpolated_Output * Ch1_Pulse_Output_Active; /* implements pulse modulation*/
Interpolated_Output = Interpolated_Output * Ch1_AM_Temp;
Interpolated_Output = Interpolated_Output  >> 24;
WriteSPI2((int) Interpolated_Output);

/* right shift to address LUT */
Delta_Phase_Acc_Ch2 = (Phase_Acc_Ch2  + Ch2_Phase_Delta);
Phase_Reg_Temp = Delta_Phase_Acc_Ch2 >> 20;
Delta_Phase_Acc_Ch2 = Delta_Phase_Acc_Ch2 & 0xfffff;
Next_Phase_Reg_Temp = Phase_Reg_Temp + 1;
if (Next_Phase_Reg_Temp == 4096)
    Next_Phase_Reg_Temp = 0;/*delta Volts in step */
Interpolated_Output = (long int)Ch2_LUT[Phase_Reg_Temp];
Temp_Interpolated_Output = (Ch2_LUT[Next_Phase_Reg_Temp] - Ch2_LUT[Phase_Reg_Temp]);
Temp_Interpolated_Output = Temp_Interpolated_Output * Delta_Phase_Acc_Ch2;
Temp_Interpolated_Output = Temp_Interpolated_Output /0x100000;
Interpolated_Output = Interpolated_Output + Temp_Interpolated_Output;
/* implement amplitude scaling */
if(Ch2_Level_Mult != 0xff)
{
    Interpolated_Output *= (long int)Ch2_Level_Mult;
    Interpolated_Output = Interpolated_Output >>8;
}
Interpolated_Output = Interpolated_Output * Ch2_Pulse_Output_Active; /* implements pulse modulation*/
Interpolated_Output = Interpolated_Output * Ch2_AM_Temp;
Interpolated_Output = Interpolated_Output  >> 24;
WriteSPI2((int) Interpolated_Output);


/* set old phase countger = current for next cycle*/
Old_Phase_Acc_Ch1 = Phase_Acc_Ch1;
Old_Phase_Acc_Ch2 = Phase_Acc_Ch2;

mPORTBToggleBits(BIT_8);
//Nop();
mPORTBToggleBits(BIT_8);
//Nop();


INTClearFlag(INT_SPI2);
mPORTBToggleBits(BIT_8);
//Nop();
mPORTBToggleBits(BIT_8);
//Nop();
}

/****************************************************************************/
/*                                                                          */
/* The main DDS function is in an ISR                                       */
/* The main function simply implements the HMI and sets up parameters for   */
/* the DDS.  The following variables are stored globally to ensure they are */
/* accessible to the ISR without going through too much hassle              */
/*                                                                          */
/* DDS data storage                                                         */
/* Phase_Acc_Ch1, Phase_Acc_Ch2;    phase accumulators                      */
/* Phase_Inc_Ch1, Phase_Inc_Ch2     phase increment per clock               */
/* Cycles_On_Ch1, Cycles_On_Ch2;    number of cycles on for pulse mode      */
/* Cycles_Off_Ch1, Cycles_Off_Ch2;  number of cycles off for pulse mode     */
/* FM_Phase_Acc_Ch1, FM_Phase_Acc_Ch2; phase accumulator for FM             */
/* AM_Phase_Acc_Ch1, AM_Phase_Acc_Ch2; phase accumulator for AM             */
/****************************************************************************/
void main(void) {
char Update_Display = 1; 		/* Flag saying need to update display */
int Freq_Speed, Freq_Speed_Counter = 1; /* use this to keep track of How Fast to count Frequency */
long int Auto_Revert = Auto_Revert_Init_Val;	
					/* Use this as a counter - after 255 loops auto revert to volume mode */
char Band = 0;           		/* use to drive state machine band select */
int Step_Size = 0;        		/* Use this to implement push button steps */  
int Display_Value = 0;                  /* use this to display the current value */
int Mode = Ch_1_Mode;                  /* use this to track current mode */
char Memory_Bank = Default_Mem_Bank;    /* use this to track which memory bank is in use */
int Channel_Mem_Offset;                        /* used to implement channel 1 & 2 offset on save call */
int Sel_Fn = Choosing_Fn;		/* Use this to track current Selection */
long long Temp_Calc_Store;              /* 64 bits helps with some sums....*/
int temp;                               /* local storage */
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//STEP 1. Configure cache, wait states and peripheral bus clock
// Configure the device for maximum performance but do not change the PBDIV
// Given the options, this function will change the flash wait states, RAM
// wait state and enable prefetch cache but will not change the PBDIV.
// The PBDIV value is already set via the pragma FPBDIV option above..
SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

// configure for multi-vectored mode
INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);

/* intitialise the Micro I/O ports*/
Init_IO();
/*Allows LCD to settle*/
DelayMs(100);
Init_LCD();				/* initialise the LCD */
Update_LCD();				/* Writes Line1 and Line2 to LCD */
DelayMs(500);

/* Initialise and load parameters from EEPROM - we need these to load defaults & saved values */
Load_From_Memory(Memory_Bank); /* Read Data from ROM */

/****************************************************************************/	
/* Get reasonable initialisation, go straight to volume			*/
/****************************************************************************/	
Sel_Fn = Ch_1_Freq;
Display_Value = Data_Values.Ch_1_Freq_Sel;
Display_Data(Sel_Fn, Display_Value);

/* give things a bit to settle */
Step_Size = 0;
Key_Just_Pressed = 0;

/* initialise DDS*/
/* update the phase increment */
Temp_Calc_Store = 0x100000000; // 2^32
Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_1_Freq_Sel;
Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate * 10);  // freq is in 10th of Hz
Phase_Inc_Ch1 = (long) Temp_Calc_Store;

/* update the phase increment */
Temp_Calc_Store = 0x100000000; // 2^32
Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_2_Freq_Sel;
Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate * 10);  // freq is in 10th of Hz
Phase_Inc_Ch2 = (long) Temp_Calc_Store;

/* This is used in the ISR to scale the output */
Ch1_Level_Mult = Atten_LUT[Data_Values.Ch_1_Level_Sel];
Ch2_Level_Mult = Atten_LUT[Data_Values.Ch_2_Level_Sel];

/* initialise LUT's */
for(temp = 0; temp < 4096; temp++)
{
    Ch1_LUT[temp] = Sine_LUT[temp];
    Ch2_LUT[temp] = Sine_LUT[temp];
}

/* set up sweep variables */
Ch1_Sweep_Phase_Reg = 0; /* start with zero sweep rate*/
if (Data_Values.Ch_1_Sweep_Type_Sel ==  Sweep_Lin)
    Ch1_Sweep_On = 1;
else
    Ch1_Sweep_On = 0;

if (Data_Values.Ch_1_Freq_Sel > (10 * Data_Values.Ch_1_Sweep_Stop_Freq_Sel))
    Ch1_Sweep_Upwards = 0;
else
    Ch1_Sweep_Upwards = 1;

/* now calculate phase increment*/
Temp_Calc_Store = (long long)0x40000000000;
/* Delta freq and include factor of 10 as time is in 10ths of seconds */
Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_1_Sweep_Stop_Freq_Sel * 10) - (long long)Data_Values.Ch_1_Freq_Sel);
Temp_Calc_Store = Temp_Calc_Store/((long long)Sample_Rate * (long long)Sample_Rate * (long long)Data_Values.Ch_1_Sweep_Time_Sel);
Ch1_Sweep_Phase_Inc = Temp_Calc_Store;
Temp_Calc_Store = (long long)0x40000000000;
/* Delta freq and include factor of 10 as time is in 10ths of seconds */
Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_2_Sweep_Stop_Freq_Sel * 10)- (long long)Data_Values.Ch_2_Freq_Sel);
Temp_Calc_Store = Temp_Calc_Store/((long long)Sample_Rate * (long long)Sample_Rate * (long long)Data_Values.Ch_2_Sweep_Time_Sel);
Ch2_Sweep_Phase_Inc = Temp_Calc_Store;


/* set up sweep variables */
Ch2_Sweep_Phase_Reg = 0; /* start with zero sweep rate*/
if (Data_Values.Ch_2_Sweep_Type_Sel ==  Sweep_Lin)
    Ch2_Sweep_On = 1;
else
    Ch2_Sweep_On = 0;

if (Data_Values.Ch_2_Freq_Sel > (10* Data_Values.Ch_2_Sweep_Stop_Freq_Sel))
    Ch2_Sweep_Upwards = 0;
else
    Ch2_Sweep_Upwards = 1;


/* now calculate end phase increment*/
/* remember we have a factor of 10 difference in sweep frequency units and frequency units...  */
Temp_Calc_Store = (long long)0x40000000000;
Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_1_Sweep_Stop_Freq_Sel * 10) - (long long)Data_Values.Ch_1_Freq_Sel);
Temp_Calc_Store = Temp_Calc_Store/(((long long)Sample_Rate) * 10);
Ch1_Sweep_End = Temp_Calc_Store;
Temp_Calc_Store = (long long)0x40000000000;
Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_2_Sweep_Stop_Freq_Sel * 10) - (long long)Data_Values.Ch_2_Freq_Sel);
Temp_Calc_Store = Temp_Calc_Store/(((long long)Sample_Rate) * 10);
Ch2_Sweep_End = Temp_Calc_Store;

/* initialise all the pulse stuff*/
Ch1_Pulse_Active = Data_Values.Ch_1_Pulse_Sel; /* Channel 1 and 2 Pule on */
Ch2_Pulse_Active = Data_Values.Ch_2_Pulse_Sel; /* Channel 1 and 2 Pule on */
Ch1_Pulse_Output_Active = 1; /* Channel 1 and 2 Pule Output on */
Ch2_Pulse_Output_Active = 1; /* Channel 1 and 2 Pule Output on */
Ch1_Pulse_Cycles_Active = Data_Values.Ch_1_PM_Cycles_On_Sel; /* Channel 1 and 2 Pule Output active */
Ch2_Pulse_Cycles_Active = Data_Values.Ch_2_PM_Cycles_On_Sel; /* Channel 1 and 2 Pule Output active */
Ch1_Pulse_Cycles_InActive = Data_Values.Ch_1_PM_Cycles_Off_Sel; /* Channel 1 and 2 Pule Output inactive */
Ch2_Pulse_Cycles_InActive = Data_Values.Ch_2_PM_Cycles_Off_Sel; /* Channel 1 and 2 Pule Output inactive */
Ch1_Pulse_Counter = Ch1_Pulse_Cycles_Active; /* Initialise this to ensure things work politely*/
Ch2_Pulse_Counter = Ch2_Pulse_Cycles_Active; /* Initialise this to ensure things work politely*/

/* phase offset initialisation*/
Temp_Calc_Store = (long long)0x100000000;
Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_1_Phase_Sel;
Temp_Calc_Store = Temp_Calc_Store / Phase_Max;
Ch1_Phase_Delta = (int) Temp_Calc_Store; /* Initialise this to ensure things work politely*/
Temp_Calc_Store = (long long)0x100000000;
Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_2_Phase_Sel;
Temp_Calc_Store = Temp_Calc_Store / Phase_Max;
Ch2_Phase_Delta = (int) Temp_Calc_Store; /* Initialise this to ensure things work politely*/

/* initialise the AM and FM stuff*/
Ch1_AM_On = (Data_Values.Ch_1_AM_Sel == AM_On); /* make these boolean */
Ch2_AM_On = (Data_Values.Ch_2_AM_Sel == AM_On); /* make these boolean */
Ch1_FM_On =  (Data_Values.Ch_1_FM_Sel == FM_On); /* make these boolean */
Ch2_FM_On =  (Data_Values.Ch_2_FM_Sel == FM_On); /* make these boolean */
Ch1_AM_Mult = (long) ( (long long)Data_Values.Ch_1_AM_Level_Sel * ((long long)0x7fffffff/100));
Ch2_AM_Mult = (long) ( (long long)Data_Values.Ch_2_AM_Level_Sel * ((long long)0x7fffffff/100));
Ch1_AM_Phase_Reg = 0; /* start clean */
Ch2_AM_Phase_Reg = 0; /* start clean */
Ch1_FM_Phase_Reg = 0; /* start clean */
Ch2_FM_Phase_Reg = 0; /* start clean */
/* calculate the phase increment for Ch1 AM*/
Temp_Calc_Store = 0x100000000; // 2^32
Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_1_AM_Freq_Sel;
Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz
Ch1_AM_Phase_Inc = (long) Temp_Calc_Store;
/* calculate the phase increment for Ch2 AM*/
Temp_Calc_Store = 0x100000000; // 2^32
Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_2_AM_Freq_Sel;
Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz
Ch2_AM_Phase_Inc = (long) Temp_Calc_Store;
/* calculate and load CH1 FM table here */
Temp_Calc_Store = 0x100000000; // 2^32
Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_1_FM_Excursion_Sel;
Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz - this is the peak
for(temp = 0; temp < 2048; temp++)
    {
        /* watch out - you are hard coding for the scale different between the SINE lut and FM LUT*/
        Ch1_FM_LUT[temp] = (long)(((long long) Temp_Calc_Store * (long long) Sine_LUT[temp*2]) / 0x1000000);
    }
/* calculate and load CH2 FM table here */
Temp_Calc_Store = 0x100000000; // 2^32
Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_2_FM_Excursion_Sel;
Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz - this is the peak
for(temp = 0; temp < 2048; temp++)
    {
        /* watch out - you are hard coding for the scale different between the SINE lut and FM LUT*/
        Ch2_FM_LUT[temp] = (long)(((long long) Temp_Calc_Store * (long long) Sine_LUT[temp*2]) / 0x1000000);
    }

/* calculate the phase increment for Ch1 FM*/
Temp_Calc_Store = 0x100000000; // 2^32
Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_1_FM_Freq_Sel;
Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz
Ch1_FM_Phase_Inc = (long) Temp_Calc_Store;
/* calculate the phase increment for Ch2 FM*/
Temp_Calc_Store = 0x100000000; // 2^32
Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_2_FM_Freq_Sel;
Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz
Ch2_FM_Phase_Inc = (long) Temp_Calc_Store;


/****************************************************************************/	
/* This sets up the test ISR 					  	    */
/* Should be about 500us cycle 						    */
/****************************************************************************/	
/* We will eventually want to be able to set the following: */
/* IEC0 bit 4, Timer1 */
/* IEC0 bit 9, Timer2 */
/* IEC1 bit 2, USB */
/* IEC1 bit 5, SPI1TX */
//turn t1 off
T1CON = 0X0;
// Clear the timer
TMR1 = 0x0000;
//Sets period register
PR1 = 0xFFFF;
// Open the timer
// T1_TICK = 1/256 seconds - we want 500uS
// T1_TICK/4 is what we want...
OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_8, (T1_TICK/4));
// set up the timer interrupt with a priority of 2
INTSetVectorPriority(INT_TIMER_1_VECTOR, INT_PRIORITY_LEVEL_4);
INTSetVectorSubPriority(INT_TIMER_1_VECTOR, INT_SUB_PRIORITY_LEVEL_3);
ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4 | T1_INT_SUB_PRIOR_3);
INTClearFlag(INT_T1);
INTEnable(INT_T1, 1);

//Enable SPI1 interrupt.
mSPI2IntDisable();
INTClearFlag(INT_SPI2);
ConfigIntSPI2(SPI_FAULT_INT_DIS | SPI_TX_INT_EN | SPI_RX_INT_DIS | SPI_INT_SUB_PRI_0 | SPI_INT_PRI_3);
SpiChnClrIntFlags(2);
INTEnableInterrupts();

/* the main loop that runs the HMI - when it is not being interrupted!*/
do
{
/* Look after changing modes */

    mPORTBToggleBits(BIT_10);
    Nop();
	if (Key_Just_Pressed)
	{
		Update_Display = 1;
		Auto_Revert = Auto_Revert_Init_Val;		/* If key pressed, reset the counter */

		/********************************************************************/
		/* Currently in Change Mode - act on legal button presses 			*/
		/********************************************************************/
		if (Sel_Fn == Choosing_Fn)
		{
			/* Up or Down*/
			if (Up_Keys || Down_Keys)
			{
				if (Up_Keys)
					Mode += 1;
				else
					Mode -= 1;
				/* check for wrap around */
				if (Mode > Load_Mode)
					Mode = Ch_1_Mode;
				if (Mode < Ch_1_Mode)
					Mode = Load_Mode;
			}

			/* Select Key hit */
			else if Sel_Key
			{
				Sel_Fn = Mode*10;
			}
			/* Exit */
			else if Exit_Key
			{
				Sel_Fn = Choosing_Fn;
			}
			else;
			
			if (Sel_Fn == Choosing_Fn)
				Display_Value = Mode;
                        else if (Sel_Fn == Ch_1_Freq)
				Display_Value = Data_Values.Ch_1_Freq_Sel;
                        else if (Sel_Fn == Ch_2_Freq)
				Display_Value = Data_Values.Ch_2_Freq_Sel;
			else if (Sel_Fn == Save_Sel_Fn)
				Display_Value = Memory_Bank;
			else if (Sel_Fn == Load_Sel_Fn)
				Display_Value = Memory_Bank;
			else;
		}

		/********************************************************************/
		/* Currently in Ch_1_Freq Function - act on legal button presses*/
		/********************************************************************/
                else if (Sel_Fn == Ch_1_Freq)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/
   
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_1_Freq_Sel = Data_Values.Ch_1_Freq_Sel - Freq_Speed;
				if ((Data_Values.Ch_1_Freq_Sel) <  Freq_Min)
                                        Data_Values.Ch_1_Freq_Sel =  Freq_Min;
				Display_Value = Data_Values.Ch_1_Freq_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_1_Freq_Sel = Data_Values.Ch_1_Freq_Sel + Freq_Speed;
				if ((Data_Values.Ch_1_Freq_Sel) >  Freq_Max)
                                        Data_Values.Ch_1_Freq_Sel =  Freq_Max;
				Display_Value = Data_Values.Ch_1_Freq_Sel;
			}
			else if Sel_Key
			{
				Sel_Fn = Freq_Speed_Counter_1_fn;
                                HDWordWriteSPI(Freq_Offset, Data_Values.Ch_1_Freq_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Freq_Speed_Counter;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
                                HDWordWriteSPI(Freq_Offset, Data_Values.Ch_1_Freq_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Mode;
			}
			else;
		/* update the phase increment */
                Temp_Calc_Store = 0x100000000; // 2^32
                Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_1_Freq_Sel;
                Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate * 10);  // freq is in 10th of Hz
                Phase_Inc_Ch1 = (long) Temp_Calc_Store;
                }

      		/********************************************************************/
		/* Currently in Ch_2_Freq Function - act on legal button presses    */
		/********************************************************************/
                else if (Sel_Fn == Ch_2_Freq)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; /* reset it...*/

			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_2_Freq_Sel = Data_Values.Ch_2_Freq_Sel - Freq_Speed;
				if ((Data_Values.Ch_2_Freq_Sel) <  Freq_Min)
                                        Data_Values.Ch_2_Freq_Sel =  Freq_Min;
				Display_Value = Data_Values.Ch_2_Freq_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_2_Freq_Sel = Data_Values.Ch_2_Freq_Sel + Freq_Speed;
				if ((Data_Values.Ch_2_Freq_Sel) >  Freq_Max)
                                        Data_Values.Ch_2_Freq_Sel =  Freq_Max;
				Display_Value = Data_Values.Ch_2_Freq_Sel;
			}
			else if Sel_Key
			{
				Sel_Fn = Freq_Speed_Counter_2_fn;
                                HDWordWriteSPI((Channel_Set_Array_Size + Freq_Offset), Data_Values.Ch_2_Freq_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Freq_Speed_Counter;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
		/* update the phase increment */
                Temp_Calc_Store = 0x100000000; // 2^32
                Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_2_Freq_Sel;
                Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate * 10);  // freq is in 10th of Hz
                Phase_Inc_Ch2 = (long) Temp_Calc_Store;
                }

      		/********************************************************************/
		/* Currently in Freq_Speed_Counter Function - act on legal button presses*/
		/********************************************************************/
                else if (Sel_Fn == Freq_Speed_Counter_1_fn)
		{
			/* Down */
			if Down_Keys
			{
				Freq_Speed_Counter = Freq_Speed_Counter/10;
				if ((Freq_Speed_Counter) <  Min_Freq_Steps)
                                        Freq_Speed_Counter =  Min_Freq_Steps;
				Display_Value = Freq_Speed_Counter;
			}
			/* Up */
			else if Up_Keys
			{
				Freq_Speed_Counter = Freq_Speed_Counter * 10;
				if ((Freq_Speed_Counter) >  Max_Freq_Steps)
                                        Freq_Speed_Counter =  Max_Freq_Steps;
				Display_Value = Freq_Speed_Counter;
			}
			else if Sel_Key
			{
				Sel_Fn = Ch_1_Level;
				Display_Value = Data_Values.Ch_1_Level_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Ch_1_Freq;
				Display_Value = Data_Values.Ch_1_Freq_Sel;
                        }
			else;
		}

       		/********************************************************************/
		/* Currently in Freq_Speed_Counter Function - act on legal button presses*/
		/********************************************************************/
                else if (Sel_Fn == Freq_Speed_Counter_2_fn)
		{
			/* Down */
			if Down_Keys
			{
				Freq_Speed_Counter = Freq_Speed_Counter/10;
				if ((Freq_Speed_Counter) <  Min_Freq_Steps)
                                        Freq_Speed_Counter =  Min_Freq_Steps;
				Display_Value = Freq_Speed_Counter;
			}
			/* Up */
			else if Up_Keys
			{
				Freq_Speed_Counter = Freq_Speed_Counter * 10;
				if ((Freq_Speed_Counter) >  Max_Freq_Steps)
                                        Freq_Speed_Counter =  Max_Freq_Steps;
				Display_Value = Freq_Speed_Counter;
			}
			else if Sel_Key
			{
				Sel_Fn = Ch_2_Level;
				Display_Value = Data_Values.Ch_2_Level_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Ch_2_Freq;
				Display_Value = Data_Values.Ch_2_Freq_Sel;
                        }
			else;
		}

                /********************************************************************/
		/* Currently in Ch_1_Level Function - act on legal button presses   */
		/********************************************************************/
                else if (Sel_Fn == Ch_1_Level) 
		{
                    if(Fast_Change != 1)
                        Fast_Change = 3;
			/* Down */
			if Down_Keys
			{
				if (Data_Values.Ch_1_Level_Sel >=  (Level_Min + Fast_Change))
				{
					Data_Values.Ch_1_Level_Sel = Data_Values.Ch_1_Level_Sel - Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_1_Level_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Ch_1_Level_Sel <= (Level_Max - Fast_Change))
				{
					Data_Values.Ch_1_Level_Sel = Data_Values.Ch_1_Level_Sel + Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_1_Level_Sel;
			}
			else if Sel_Key
			{
				Sel_Fn = Ch_1_Waveform;
                                HDWordWriteSPI(Level_Offset, Data_Values.Ch_1_Level_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Waveform_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                /* This is used in the ISR to scale the output */
                Ch1_Level_Mult = Atten_LUT[Data_Values.Ch_1_Level_Sel];
		}

                /********************************************************************/
		/* Currently in Ch_2_Level Function - act on legal button presses   */
		/********************************************************************/
                else if (Sel_Fn == Ch_2_Level)
		{
                    if(Fast_Change != 1)
                        Fast_Change = 3;
			/* Down */
			if Down_Keys
			{
				if (Data_Values.Ch_2_Level_Sel >=  (Level_Min + Fast_Change))
				{
					Data_Values.Ch_2_Level_Sel = Data_Values.Ch_2_Level_Sel - Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_2_Level_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Ch_2_Level_Sel <= (Level_Max - Fast_Change))
				{
					Data_Values.Ch_2_Level_Sel = Data_Values.Ch_2_Level_Sel + Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_2_Level_Sel;
			}
			else if Sel_Key
			{
				Sel_Fn = Ch_2_Waveform;
                                HDWordWriteSPI((Channel_Set_Array_Size + Level_Offset), Data_Values.Ch_2_Level_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_2_Waveform_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                /* This is used in the ISR to scale the output */
                Ch2_Level_Mult = Atten_LUT[Data_Values.Ch_2_Level_Sel];
                }

                /***********************************************************************/
		/* Currently in Ch_1_Waveform Function - act on legal button presses   */
		/***********************************************************************/
                else if (Sel_Fn == Ch_1_Waveform)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_1_Waveform_Sel = Data_Values.Ch_1_Waveform_Sel - 1;
				if ((Data_Values.Ch_1_Waveform_Sel) <  Waveform_Min)
                                        Data_Values.Ch_1_Waveform_Sel =  Waveform_Max;
				Display_Value = Data_Values.Ch_1_Waveform_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_1_Waveform_Sel = Data_Values.Ch_1_Waveform_Sel + 1;
				if ((Data_Values.Ch_1_Waveform_Sel) > Waveform_Max)
                                    Data_Values.Ch_1_Waveform_Sel = Waveform_Min;
    				Display_Value = Data_Values.Ch_1_Waveform_Sel;
			}
			else if Sel_Key
			{
                            if (Data_Values.Ch_1_Waveform_Sel == Pulse_Sel)
                            {
                                Sel_Fn = Ch_1_Ratio;
                                HDWordWriteSPI(Waveform_Offset, Data_Values.Ch_1_Waveform_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Ratio_Sel;
                            }
                            else
                            {
                                Sel_Fn = Ch_1_Sweep;
                                HDWordWriteSPI(Waveform_Offset, Data_Values.Ch_1_Waveform_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Sweep_Type_Sel;
                            }
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;


                        if (Data_Values.Ch_1_Waveform_Sel == Sine_Sel)
                            {
                                /* initialise LUT's */
                                for(temp = 0; temp < 4096; temp++)
                                {
                                    Ch1_LUT[temp] = Sine_LUT[temp];
                                }
                            }
                        else if (Data_Values.Ch_1_Waveform_Sel == Arb_Sel)
                            {
                                /* initialise LUT's */
                                for(temp = 0; temp < 4096; temp++)
                                {
                                    Ch1_LUT[temp] = Noise_LUT[temp];
                                }
                            }
                        else if (Data_Values.Ch_1_Waveform_Sel == Triangle_Sel)
                            {
                                /* initialise LUT's */
                                for(temp = 0; temp < 1024; temp++)
                                {
                                    Ch1_LUT[temp] = temp * 0x2000;
                                }
                                for(temp = 0; temp < 2048; temp++)
                                {
                                    Ch1_LUT[1024 + temp] = 0x7fffff - temp * 0x2000;
                                }
                                for(temp = 0; temp < 1024; temp++)
                                {
                                    Ch1_LUT[3072 + temp] = -0x7fffff + temp * 0x2000;
                                }
                            }
                        else if (Data_Values.Ch_1_Waveform_Sel == Square_Sel)
                            {
                                /* initialise LUT's */
                                for(temp = 0; temp < 2048; temp++)
                                {
                                    Ch1_LUT[temp] = +0x7fffff;
                                }
                                for(temp = 0; temp < 2048; temp++)
                                {
                                    Ch1_LUT[2048 + temp] = -0x7fffff;
                                }
                            }
                       else if (Data_Values.Ch_1_Waveform_Sel == Pulse_Sel)
                            {
                                /* initialise LUT's */
                                for(temp = 0; temp < ((4096*Data_Values.Ch_1_Ratio_Sel)/100); temp++)
                                {
                                    Ch1_LUT[temp] = 0x7fffff;
                                }
                                for(temp = ((4096*Data_Values.Ch_1_Ratio_Sel)/100); temp < 4096; temp++)
                                {
                                    Ch1_LUT[temp] = -0x7fffff;
                                }
                            }
               }

                /***********************************************************************/
		/* Currently in Ch_2_Waveform Function - act on legal button presses   */
		/***********************************************************************/
                else if (Sel_Fn == Ch_2_Waveform)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_2_Waveform_Sel = Data_Values.Ch_2_Waveform_Sel - 1;
				if ((Data_Values.Ch_2_Waveform_Sel) <  Waveform_Min)
                                        Data_Values.Ch_2_Waveform_Sel =  Waveform_Max;
				Display_Value = Data_Values.Ch_2_Waveform_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_2_Waveform_Sel = Data_Values.Ch_2_Waveform_Sel + 1;
				if ((Data_Values.Ch_2_Waveform_Sel) > Waveform_Max)
                                    Data_Values.Ch_2_Waveform_Sel = Waveform_Min;
    				Display_Value = Data_Values.Ch_2_Waveform_Sel;
			}
			else if Sel_Key
			{
                            if (Data_Values.Ch_2_Waveform_Sel == Pulse_Sel)
                            {
                                Sel_Fn = Ch_2_Ratio;
                                HDWordWriteSPI((Channel_Set_Array_Size + Waveform_Offset), Data_Values.Ch_2_Waveform_Sel);
				Display_Value = Data_Values.Ch_2_Ratio_Sel;
                            }
                            else
                            {
                                Sel_Fn = Ch_2_Sweep;
                                HDWordWriteSPI((Channel_Set_Array_Size + Waveform_Offset), Data_Values.Ch_2_Waveform_Sel);
				Display_Value = Data_Values.Ch_2_Sweep_Type_Sel;
                            }
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
                                HDWordWriteSPI(Waveform_Offset, Data_Values.Ch_2_Waveform_Sel);
				Display_Value = Mode;
			}
			else;

                        if (Data_Values.Ch_2_Waveform_Sel == Sine_Sel)
                            {
                                /* initialise LUT's */
                                for(temp = 0; temp < 4096; temp++)
                                {
                                    Ch2_LUT[temp] = Sine_LUT[temp];
                                }
                            }
                        else if (Data_Values.Ch_2_Waveform_Sel == Arb_Sel)
                            {
                                /* initialise LUT's */
                                for(temp = 0; temp < 4096; temp++)
                                {
                                    Ch2_LUT[temp] = Noise_LUT[temp];
                                }
                            }
                        else if (Data_Values.Ch_2_Waveform_Sel == Triangle_Sel)
                            {
                                /* initialise LUT's */
                                for(temp = 0; temp < 1024; temp++)
                                {
                                    Ch2_LUT[temp] = temp * 0x2000;
                                }
                                for(temp = 0; temp < 2048; temp++)
                                {
                                    Ch2_LUT[1024 + temp] = 0x7fffff - temp * 0x2000;
                                }
                                for(temp = 0; temp < 1024; temp++)
                                {
                                    Ch2_LUT[3072 + temp] = -0x7fffff + temp * 0x2000;
                                }
                            }
                        else if (Data_Values.Ch_2_Waveform_Sel == Square_Sel)
                            {
                                /* initialise LUT's */
                                for(temp = 0; temp < 2048; temp++)
                                {
                                    Ch2_LUT[temp] = 0x7fffff;
                                }
                                for(temp = 0; temp < 2048; temp++)
                                {
                                    Ch2_LUT[2048 + temp] = -0x7fffff;
                                }
                            }
                         else if (Data_Values.Ch_2_Waveform_Sel == Pulse_Sel)
                            {
                                /* initialise LUT's */
                                for(temp = 0; temp < ((4096*Data_Values.Ch_2_Ratio_Sel)/100); temp++)
                                {
                                    Ch2_LUT[temp] = 0x7fffff;
                                }
                                for(temp = ((4096*Data_Values.Ch_2_Ratio_Sel)/100); temp < 4096; temp++)
                                {
                                    Ch2_LUT[temp] = -0x7fffff;
                                }
                            }
                }

                /********************************************************************/
		/* Currently in Ch_1_Ratio Function - act on legal button presses   */
		/********************************************************************/
                else if (Sel_Fn == Ch_1_Ratio)
		{
                    if(Fast_Change != 1)
                        Fast_Change = 10;			/* Down */
			if Down_Keys
			{
				if (Data_Values.Ch_1_Ratio_Sel >=  (Ratio_Min + Fast_Change))
				{
					Data_Values.Ch_1_Ratio_Sel = Data_Values.Ch_1_Ratio_Sel - Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_1_Ratio_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Ch_1_Ratio_Sel <= (Ratio_Max - Fast_Change))
				{
					Data_Values.Ch_1_Ratio_Sel = Data_Values.Ch_1_Ratio_Sel + Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_1_Ratio_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_Sweep;
                                HDWordWriteSPI(Ratio_Offset, Data_Values.Ch_1_Ratio_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Sweep_Type_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                         /* initialise LUT's */
                         for(temp = 0; temp < ((4096*Data_Values.Ch_1_Ratio_Sel)/100); temp++)
                                {
                                    Ch1_LUT[temp] = 0x7fffff;
                                }
                         for(temp = ((4096*Data_Values.Ch_1_Ratio_Sel)/100); temp < 4096; temp++)
                                {
                                    Ch1_LUT[temp] = -0x7fffff;
                                }
		}

                /********************************************************************/
		/* Currently in Ch_2_Ratio Function - act on legal button presses   */
		/********************************************************************/
                else if (Sel_Fn == Ch_2_Ratio)
		{
                    if(Fast_Change != 1)
                        Fast_Change = 10;			/* Down */
			if Down_Keys
			{
				if (Data_Values.Ch_2_Ratio_Sel >=  (Ratio_Min + Fast_Change))
				{
					Data_Values.Ch_2_Ratio_Sel = Data_Values.Ch_2_Ratio_Sel - Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_2_Ratio_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Ch_2_Ratio_Sel <= (Ratio_Max - Fast_Change))
				{
					Data_Values.Ch_2_Ratio_Sel = Data_Values.Ch_2_Ratio_Sel + Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_2_Ratio_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_2_Sweep;
                                HDWordWriteSPI((Channel_Set_Array_Size + Ratio_Offset), Data_Values.Ch_2_Ratio_Sel);
				Display_Value = Data_Values.Ch_2_Sweep_Type_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                          /* initialise LUT's */
                         for(temp = 0; temp < ((4096*Data_Values.Ch_2_Ratio_Sel)/100); temp++)
                                {
                                    Ch2_LUT[temp] = 0x7fffff;
                                }
                         for(temp = ((4096*Data_Values.Ch_2_Ratio_Sel)/100); temp < 4096; temp++)
                                {
                                    Ch2_LUT[temp] = -0x7fffff;
                                }
		}

                /********************************************************************/
		/* Currently in Ch_1_Sweep Function - act on legal button presses   */
		/********************************************************************/
                else if (Sel_Fn == Ch_1_Sweep)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_1_Sweep_Type_Sel = Data_Values.Ch_1_Sweep_Type_Sel - 1;
				if ((Data_Values.Ch_1_Sweep_Type_Sel) <  Sweep_Off )
                                    Data_Values.Ch_1_Sweep_Type_Sel =  Sweep_Lin;
				Display_Value = Data_Values.Ch_1_Sweep_Type_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_1_Sweep_Type_Sel = Data_Values.Ch_1_Sweep_Type_Sel + 1;
				if ((Data_Values.Ch_1_Sweep_Type_Sel) >  Sweep_Lin )
                                    Data_Values.Ch_1_Sweep_Type_Sel =  Sweep_Off;
				Display_Value = Data_Values.Ch_1_Sweep_Type_Sel;
			}
			else if Sel_Key
			{
                            if (Data_Values.Ch_1_Sweep_Type_Sel == Sweep_Off)
                            {
                                Ch1_Sweep_Phase_Reg = 0;
                                Sel_Fn = Ch_1_Pulse;
                                HDWordWriteSPI(Sweep_Type_Offset, Data_Values.Ch_1_Sweep_Type_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Pulse_Sel;
                            }
                            else
                            {
                                Sel_Fn = Ch_1_Sweep_Stop;
                                HDWordWriteSPI(Sweep_Type_Offset, Data_Values.Ch_1_Sweep_Type_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Sweep_Stop_Freq_Sel;
                            }
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                        /* set up sweep variables */

                        if (Data_Values.Ch_1_Sweep_Type_Sel ==  Sweep_Lin)
                            Ch1_Sweep_On = 1;
                        else
                            Ch1_Sweep_On = 0;
		}

                /********************************************************************/
		/* Currently in Ch_2_Sweep Function - act on legal button presses   */
		/********************************************************************/
                else if (Sel_Fn == Ch_2_Sweep)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_2_Sweep_Type_Sel = Data_Values.Ch_2_Sweep_Type_Sel - 1;
				if ((Data_Values.Ch_2_Sweep_Type_Sel) <  Sweep_Off )
                                    Data_Values.Ch_2_Sweep_Type_Sel =  Sweep_Lin;
				Display_Value = Data_Values.Ch_2_Sweep_Type_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_2_Sweep_Type_Sel = Data_Values.Ch_2_Sweep_Type_Sel + 1;
				if ((Data_Values.Ch_2_Sweep_Type_Sel) >  Sweep_Lin )
                                    Data_Values.Ch_2_Sweep_Type_Sel =  Sweep_Off;
				Display_Value = Data_Values.Ch_2_Sweep_Type_Sel;
			}
			else if Sel_Key
			{
                            if (Data_Values.Ch_2_Sweep_Type_Sel == Sweep_Off)
                            {
                                Ch2_Sweep_Phase_Reg = 0;
                                Sel_Fn = Ch_2_Pulse;
                                HDWordWriteSPI((Channel_Set_Array_Size + Sweep_Type_Offset), Data_Values.Ch_2_Sweep_Type_Sel);
				Display_Value = Data_Values.Ch_2_Pulse_Sel;
                            }
                            else
                            {
                                Sel_Fn = Ch_2_Sweep_Stop;
                                HDWordWriteSPI((Channel_Set_Array_Size + Sweep_Type_Offset), Data_Values.Ch_2_Sweep_Type_Sel);
				Display_Value = Data_Values.Ch_2_Sweep_Stop_Freq_Sel;
                            }
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        if (Data_Values.Ch_2_Sweep_Type_Sel ==  Sweep_Lin)
                            Ch2_Sweep_On = 1;
                        else
                            Ch2_Sweep_On = 0;
                }

                /*************************************************************************/
		/* Currently in Ch_1_Sweep_Stop Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_Sweep_Stop)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; 

			if Down_Keys
			{
				if (Data_Values.Ch_1_Sweep_Stop_Freq_Sel >=  (Sweep_Stop_Min + Freq_Speed))
				{
					Data_Values.Ch_1_Sweep_Stop_Freq_Sel = Data_Values.Ch_1_Sweep_Stop_Freq_Sel - Freq_Speed;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_1_Sweep_Stop_Freq_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Ch_1_Sweep_Stop_Freq_Sel <= (Sweep_Stop_Max - Freq_Speed))
				{
					Data_Values.Ch_1_Sweep_Stop_Freq_Sel = Data_Values.Ch_1_Sweep_Stop_Freq_Sel + Freq_Speed;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_1_Sweep_Stop_Freq_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_Sweep_Time;
                                HDWordWriteSPI(Sweep_Stop_Freq_Offset, Data_Values.Ch_1_Sweep_Stop_Freq_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Sweep_Time_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* are we sweeping up or down? Remember scaling of sweep is 1/10 of main freq...*/
                        if (Data_Values.Ch_1_Freq_Sel > (10 * Data_Values.Ch_1_Sweep_Stop_Freq_Sel))
                            Ch1_Sweep_Upwards = 0;
                        else
                             Ch1_Sweep_Upwards = 1;
                        /* now calculate phase increment*/
                        Temp_Calc_Store = (long long)0x40000000000;
                        /* Delta freq and include factor of 10 as time is in 10ths of seconds */
                        Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_1_Sweep_Stop_Freq_Sel * 10) - (long long)Data_Values.Ch_1_Freq_Sel);
                        Temp_Calc_Store = Temp_Calc_Store/((long long)Sample_Rate * (long long)Sample_Rate * (long long)Data_Values.Ch_1_Sweep_Time_Sel);
                        Ch1_Sweep_Phase_Inc = Temp_Calc_Store;
                        /* and where to stop the sweep*/
                        Temp_Calc_Store = (long long)0x40000000000;
                        Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_1_Sweep_Stop_Freq_Sel * 10)- (long long)Data_Values.Ch_1_Freq_Sel);
                        Temp_Calc_Store = Temp_Calc_Store/(((long long)Sample_Rate) * 10);
                        Ch1_Sweep_End = Temp_Calc_Store;
                }


                /*************************************************************************/
		/* Currently in Ch_1_Sweep_Time Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_Sweep_Time)
		{
                    if(Fast_Change > 1)
                        Freq_Speed =  10;  /* boost speed */
                    else
                        Freq_Speed = 1; /* reset it...*/

			if Down_Keys
			{
				if (Data_Values.Ch_1_Sweep_Time_Sel >=  (Sweep_Time_Min + Freq_Speed))
				{
					Data_Values.Ch_1_Sweep_Time_Sel = Data_Values.Ch_1_Sweep_Time_Sel - Freq_Speed;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_1_Sweep_Time_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Ch_1_Sweep_Time_Sel <= (Sweep_Time_Max - Freq_Speed))
				{
					Data_Values.Ch_1_Sweep_Time_Sel = Data_Values.Ch_1_Sweep_Time_Sel + Freq_Speed;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_1_Sweep_Time_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_Pulse;
                                HDWordWriteSPI(Sweep_Time_Offset, Data_Values.Ch_1_Sweep_Time_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Pulse_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;


                        /* now calculate phase increment*/
                        Temp_Calc_Store = (long long)0x40000000000;
                        /* Delta freq and include factor of 10 as time is in 10ths of seconds */
                        Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_1_Sweep_Stop_Freq_Sel * 10) - (long long)Data_Values.Ch_1_Freq_Sel);
                        Temp_Calc_Store = Temp_Calc_Store/((long long)Sample_Rate * (long long)Sample_Rate * (long long)Data_Values.Ch_1_Sweep_Time_Sel);
                        Ch1_Sweep_Phase_Inc = Temp_Calc_Store;
                        /* and where to stop the sweep*/
                        Temp_Calc_Store = (long long)0x40000000000;
                        Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_1_Sweep_Stop_Freq_Sel * 10) - (long long)Data_Values.Ch_1_Freq_Sel);
                        Temp_Calc_Store = Temp_Calc_Store/(((long long)Sample_Rate) * 10);
                        Ch1_Sweep_End = Temp_Calc_Store;
		}

                /*************************************************************************/
		/* Currently in Ch_2_Sweep_Stop Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_Sweep_Stop)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = Freq_Speed_Counter * 10;  /* boost speed */
                    else
                        Freq_Speed = Freq_Speed_Counter; 

			if Down_Keys
			{
				if (Data_Values.Ch_2_Sweep_Stop_Freq_Sel >=  (Sweep_Stop_Min + Freq_Speed))
				{
					Data_Values.Ch_2_Sweep_Stop_Freq_Sel = Data_Values.Ch_2_Sweep_Stop_Freq_Sel - Freq_Speed;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_2_Sweep_Stop_Freq_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Ch_2_Sweep_Stop_Freq_Sel <= (Sweep_Stop_Max - Freq_Speed))
				{
					Data_Values.Ch_2_Sweep_Stop_Freq_Sel = Data_Values.Ch_2_Sweep_Stop_Freq_Sel + Freq_Speed;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_2_Sweep_Stop_Freq_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_2_Sweep_Time;
                                HDWordWriteSPI((Channel_Set_Array_Size + Sweep_Stop_Freq_Offset), Data_Values.Ch_2_Sweep_Stop_Freq_Sel);
				Display_Value = Data_Values.Ch_2_Sweep_Time_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* determine if the sweep is upwards of downwards - remember scaling of sewwp is 1.10 of main freq*/
                        if (Data_Values.Ch_2_Freq_Sel > (10 * Data_Values.Ch_2_Sweep_Stop_Freq_Sel))
                            Ch2_Sweep_Upwards = 0;
                        else
                            Ch2_Sweep_Upwards = 1;

                        /* reset the phase offset register */
                        Ch2_Sweep_Phase_Reg = 0;

                        /* now calculate phase increment*/
                        Temp_Calc_Store = (long long)0x40000000000;
                        /* Delta freq and include factor of 10 as time is in 10ths of seconds */
                        Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_2_Sweep_Stop_Freq_Sel * 10) - (long long)Data_Values.Ch_2_Freq_Sel);
                        Temp_Calc_Store = Temp_Calc_Store/((long long)Sample_Rate * (long long)Sample_Rate * (long long)Data_Values.Ch_2_Sweep_Time_Sel);
                        Ch2_Sweep_Phase_Inc = Temp_Calc_Store;
                        /* and where to stop the sweep*/
                        Temp_Calc_Store = (long long)0x40000000000;
                        Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_2_Sweep_Stop_Freq_Sel * 10)- (long long)Data_Values.Ch_2_Freq_Sel);
                        Temp_Calc_Store = Temp_Calc_Store/(((long long)Sample_Rate) * 10);
                        Ch2_Sweep_End = Temp_Calc_Store;

                }

                /*************************************************************************/
		/* Currently in Ch_2_Sweep_Time Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_Sweep_Time)
		{
                    if(Fast_Change > 1)
                        Freq_Speed = 10;  /* boost speed */
                    else
                        Freq_Speed = 1; /* reset it...*/

			if Down_Keys
			{
				if (Data_Values.Ch_2_Sweep_Time_Sel >=  (Sweep_Time_Min + Freq_Speed))
				{
					Data_Values.Ch_2_Sweep_Time_Sel = Data_Values.Ch_2_Sweep_Time_Sel - Freq_Speed;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_2_Sweep_Time_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Ch_2_Sweep_Time_Sel <= (Sweep_Time_Max - Freq_Speed))
				{
					Data_Values.Ch_2_Sweep_Time_Sel = Data_Values.Ch_2_Sweep_Time_Sel + Freq_Speed;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_2_Sweep_Time_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_2_Pulse;
                                HDWordWriteSPI((Channel_Set_Array_Size + Sweep_Time_Offset), Data_Values.Ch_2_Sweep_Time_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_2_Pulse_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* reset the phase offset register */
                        Ch2_Sweep_Phase_Reg = 0;

                        /* now calculate phase increment*/
                        Temp_Calc_Store = (long long)0x40000000000;
                        /* Delta freq and include factor of 10 as time is in 10ths of seconds */
                        Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_2_Sweep_Stop_Freq_Sel * 10) - (long long)Data_Values.Ch_2_Freq_Sel);
                        Temp_Calc_Store = Temp_Calc_Store/((long long)Sample_Rate * (long long)Sample_Rate * (long long)Data_Values.Ch_2_Sweep_Time_Sel);
                        Ch2_Sweep_Phase_Inc = Temp_Calc_Store;
                        /* and where to stop the sweep*/
                        Temp_Calc_Store = (long long)0x40000000000;
                        Temp_Calc_Store = Temp_Calc_Store * (((long long)Data_Values.Ch_2_Sweep_Stop_Freq_Sel * 10)- (long long)Data_Values.Ch_2_Freq_Sel);
                        Temp_Calc_Store = Temp_Calc_Store/(((long long)Sample_Rate) * 10);
                        Ch2_Sweep_End = Temp_Calc_Store;
		}

                /*************************************************************************/
		/* Currently in Ch_1_Pulse Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_Pulse)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_1_Pulse_Sel = Data_Values.Ch_1_Pulse_Sel - 1;
				if ((Data_Values.Ch_1_Pulse_Sel) <  Pulse_Off )
                                    Data_Values.Ch_1_Pulse_Sel =  Pulse_On;
				Display_Value = Data_Values.Ch_1_Pulse_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_1_Pulse_Sel = Data_Values.Ch_1_Pulse_Sel + 1;
				if ((Data_Values.Ch_1_Pulse_Sel) >  Pulse_On )
                                    Data_Values.Ch_1_Pulse_Sel =  Pulse_Off;
				Display_Value = Data_Values.Ch_1_Pulse_Sel;
			}
			else if Sel_Key
			{
                            if (Data_Values.Ch_1_Pulse_Sel == Pulse_Off)
                            {
                                Sel_Fn = Ch_1_Phase;
                                HDWordWriteSPI(Pulse_Offset, Data_Values.Ch_1_Pulse_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Phase_Sel;
                            }
                            else
                            {
                                Sel_Fn = Ch_1_PM_Cycles_On;
                                HDWordWriteSPI(Pulse_Offset, Data_Values.Ch_1_Pulse_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_PM_Cycles_On_Sel;
                            }
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* Update the direct RAM variable for this... */
	                Ch1_Pulse_Active = Data_Values.Ch_1_Pulse_Sel; /* Channel 1 and 2 Pule on */
                }

                /*************************************************************************/
		/* Currently in Ch_2_Pulse Function - act on legal button presses        */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_Pulse)
		{
			/* Down */
			if Down_Keys
			{
				Data_Values.Ch_2_Pulse_Sel = Data_Values.Ch_2_Pulse_Sel - 1;
				if ((Data_Values.Ch_2_Pulse_Sel) <  Pulse_Off )
                                    Data_Values.Ch_2_Pulse_Sel =  Pulse_On;
				Display_Value = Data_Values.Ch_2_Pulse_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				Data_Values.Ch_2_Pulse_Sel = Data_Values.Ch_2_Pulse_Sel + 1;
				if ((Data_Values.Ch_2_Pulse_Sel) >  Pulse_On )
                                    Data_Values.Ch_2_Pulse_Sel =  Pulse_Off;
				Display_Value = Data_Values.Ch_2_Pulse_Sel;
			}
			else if Sel_Key
			{
                            if (Data_Values.Ch_2_Pulse_Sel == Pulse_Off)
                            {
                                Sel_Fn = Ch_2_Phase;
                                HDWordWriteSPI((Channel_Set_Array_Size + Pulse_Offset), Data_Values.Ch_2_Pulse_Sel);
				Display_Value = Data_Values.Ch_2_Phase_Sel;
                            }
                            else
                            {
                                Sel_Fn = Ch_2_PM_Cycles_On;
                                HDWordWriteSPI((Channel_Set_Array_Size + Pulse_Offset), Data_Values.Ch_2_Pulse_Sel);
				Display_Value = Data_Values.Ch_2_PM_Cycles_On_Sel;
                            }
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* update the direct RAM variable for this*/
                        Ch2_Pulse_Active = Data_Values.Ch_2_Pulse_Sel; /* Channel 1 and 2 Pule on */
		}


                /*************************************************************************/
		/* Currently in Ch_1_PM_Cycles_On Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_PM_Cycles_On)
		{
                    if(Fast_Change != 1)
                        Fast_Change = 10;			/* Down */
			if Down_Keys
			{
                            Data_Values.Ch_1_PM_Cycles_On_Sel = Data_Values.Ch_1_PM_Cycles_On_Sel - Fast_Change;
                            if (Data_Values.Ch_1_PM_Cycles_On_Sel <  PM_Cycles_On_Min)
                                Data_Values.Ch_1_PM_Cycles_On_Sel =  PM_Cycles_On_Min;
                            Display_Value = Data_Values.Ch_1_PM_Cycles_On_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                            Data_Values.Ch_1_PM_Cycles_On_Sel = Data_Values.Ch_1_PM_Cycles_On_Sel + Fast_Change;
                            if (Data_Values.Ch_1_PM_Cycles_On_Sel >  PM_Cycles_On_Max)
                                Data_Values.Ch_1_PM_Cycles_On_Sel =  PM_Cycles_On_Max;
                            Display_Value = Data_Values.Ch_1_PM_Cycles_On_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_PM_Cycles_Off;
                                HDWordWriteSPI(Cycles_On_Offset, Data_Values.Ch_1_PM_Cycles_On_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_PM_Cycles_Off_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* update the direct RAM variable */
                        Ch1_Pulse_Cycles_Active = Data_Values.Ch_1_PM_Cycles_On_Sel; /* Channel 1 and 2 Pule Output active */
		}

                /*************************************************************************/
		/* Currently in Ch_2_PM_Cycles_On Function - act on legal button presses  */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_PM_Cycles_On)
		{
                    if(Fast_Change != 1)
                        Fast_Change = 10;			/* Down */
			if Down_Keys
			{
                            Data_Values.Ch_2_PM_Cycles_On_Sel = Data_Values.Ch_2_PM_Cycles_On_Sel - Fast_Change;
                            if (Data_Values.Ch_2_PM_Cycles_On_Sel <  PM_Cycles_On_Min)
                                Data_Values.Ch_2_PM_Cycles_On_Sel =  PM_Cycles_On_Min;
                            Display_Value = Data_Values.Ch_2_PM_Cycles_On_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                            Data_Values.Ch_2_PM_Cycles_On_Sel = Data_Values.Ch_2_PM_Cycles_On_Sel + Fast_Change;
                            if (Data_Values.Ch_2_PM_Cycles_On_Sel >  PM_Cycles_On_Max)
                                Data_Values.Ch_2_PM_Cycles_On_Sel =  PM_Cycles_On_Max;
                            Display_Value = Data_Values.Ch_2_PM_Cycles_On_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_2_PM_Cycles_Off;
                                HDWordWriteSPI((Channel_Set_Array_Size + Cycles_On_Offset), Data_Values.Ch_2_PM_Cycles_On_Sel);
				Display_Value = Data_Values.Ch_2_PM_Cycles_Off_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* update the direct RAM variable*/
                        Ch2_Pulse_Cycles_Active = Data_Values.Ch_2_PM_Cycles_On_Sel; /* Channel 1 and 2 Pule Output active */
		}


                /*************************************************************************/
		/* Currently in Ch_1_PM_Cycles_Off Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_PM_Cycles_Off)
		{
                    if(Fast_Change != 1)
                        Fast_Change = 10;			/* Down */
			if Down_Keys
			{
                            Data_Values.Ch_1_PM_Cycles_Off_Sel = Data_Values.Ch_1_PM_Cycles_Off_Sel - Fast_Change;
                            if (Data_Values.Ch_1_PM_Cycles_Off_Sel <  PM_Cycles_Off_Min)
                                Data_Values.Ch_1_PM_Cycles_Off_Sel =  PM_Cycles_Off_Min;
                            Display_Value = Data_Values.Ch_1_PM_Cycles_Off_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                            Data_Values.Ch_1_PM_Cycles_Off_Sel = Data_Values.Ch_1_PM_Cycles_Off_Sel + Fast_Change;
                            if (Data_Values.Ch_1_PM_Cycles_Off_Sel >  PM_Cycles_On_Max)
                                Data_Values.Ch_1_PM_Cycles_Off_Sel =  PM_Cycles_On_Max;
                            Display_Value = Data_Values.Ch_1_PM_Cycles_Off_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_Phase;
                                HDWordWriteSPI(Cycles_Off_Offset, Data_Values.Ch_1_PM_Cycles_Off_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Phase_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* update the direct RAM varibale*/
                        Ch1_Pulse_Cycles_InActive = Data_Values.Ch_1_PM_Cycles_Off_Sel; /* Channel 1 and 2 Pule Output inactive */
		}

                /****************************************************************************/
		/* Currently in Ch_2_PM_Cycles_Off Function - act on legal button presses   */
		/****************************************************************************/
                else if (Sel_Fn == Ch_2_PM_Cycles_Off)
		{
                    if(Fast_Change != 1)
                        Fast_Change = 10;			/* Down */
			if Down_Keys
			{
                            Data_Values.Ch_2_PM_Cycles_Off_Sel = Data_Values.Ch_2_PM_Cycles_Off_Sel - Fast_Change;
                            if (Data_Values.Ch_2_PM_Cycles_Off_Sel <  PM_Cycles_Off_Min)
                                Data_Values.Ch_2_PM_Cycles_Off_Sel =  PM_Cycles_Off_Min;
                            Display_Value = Data_Values.Ch_2_PM_Cycles_Off_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                            Data_Values.Ch_2_PM_Cycles_Off_Sel = Data_Values.Ch_2_PM_Cycles_Off_Sel + Fast_Change;
                            if (Data_Values.Ch_2_PM_Cycles_Off_Sel >  PM_Cycles_On_Max)
                                Data_Values.Ch_2_PM_Cycles_Off_Sel =  PM_Cycles_On_Max;
                            Display_Value = Data_Values.Ch_2_PM_Cycles_Off_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_2_Phase;
                                HDWordWriteSPI((Channel_Set_Array_Size + Cycles_Off_Offset), Data_Values.Ch_2_PM_Cycles_Off_Sel);
				Display_Value = Data_Values.Ch_2_Phase_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        /* update the direct RAM variable*/
                        Ch2_Pulse_Cycles_InActive = Data_Values.Ch_2_PM_Cycles_Off_Sel; /* Channel 1 and 2 Pule Output inactive */
		}


                /*************************************************************************/
		/* Currently in Ch_1_Phase Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_Phase)
		{
			if Down_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_1_Phase_Sel = Data_Values.Ch_1_Phase_Sel - 1;
                                else
                                    Data_Values.Ch_1_Phase_Sel = Data_Values.Ch_1_Phase_Sel - Phase_Max/180;

				if ((Data_Values.Ch_1_Phase_Sel) <  (Phase_Min ))
                                    Data_Values.Ch_1_Phase_Sel =  Phase_Max - 1;
				Display_Value = Data_Values.Ch_1_Phase_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_1_Phase_Sel = Data_Values.Ch_1_Phase_Sel + 1;
                                else
                                    Data_Values.Ch_1_Phase_Sel = Data_Values.Ch_1_Phase_Sel + Phase_Max/180;

				if ((Data_Values.Ch_1_Phase_Sel) >=  (Phase_Max ))
                                    Data_Values.Ch_1_Phase_Sel =  Phase_Min;
				Display_Value = Data_Values.Ch_1_Phase_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_AM;
                                HDWordWriteSPI(Phase_Offset, Data_Values.Ch_1_Phase_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_AM_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        Temp_Calc_Store = (long long)0x100000000;
                        Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_1_Phase_Sel;
                        Temp_Calc_Store = Temp_Calc_Store / Phase_Max;
                        Ch1_Phase_Delta = (int) Temp_Calc_Store; /* Initialise this to ensure things work politely*/
		}

                /*************************************************************************/
		/* Currently in Ch_2_Phase Function - act on legal button presses        */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_Phase)
		{
			if Down_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_2_Phase_Sel = Data_Values.Ch_2_Phase_Sel - 1;
                                else
                                    Data_Values.Ch_2_Phase_Sel = Data_Values.Ch_2_Phase_Sel - Phase_Max/180;

				if ((Data_Values.Ch_2_Phase_Sel) <  (Phase_Min ))
                                    Data_Values.Ch_2_Phase_Sel =  Phase_Max - 1;
				Display_Value = Data_Values.Ch_2_Phase_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_2_Phase_Sel = Data_Values.Ch_2_Phase_Sel + 1;
                                else
                                    Data_Values.Ch_2_Phase_Sel = Data_Values.Ch_2_Phase_Sel + Phase_Max/180;

				if ((Data_Values.Ch_2_Phase_Sel) >=  (Phase_Max ))
                                    Data_Values.Ch_2_Phase_Sel =  Phase_Min;
				Display_Value = Data_Values.Ch_2_Phase_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_2_AM;
                                HDWordWriteSPI((Channel_Set_Array_Size + Phase_Offset), Data_Values.Ch_2_Phase_Sel);
				Display_Value = Data_Values.Ch_2_AM_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;

                        Temp_Calc_Store = (long long)0x100000000;
                        Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_2_Phase_Sel;
                        Temp_Calc_Store = Temp_Calc_Store / Phase_Max;
                        Ch2_Phase_Delta = (int) Temp_Calc_Store; /* Initialise this to ensure things work politely*/
		}


                /*************************************************************************/
		/* Currently in Ch_1_AM Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_AM)
		{
			/* Down */
			if Down_Keys
			{
                            Data_Values.Ch_1_AM_Sel = Data_Values.Ch_1_AM_Sel - 1;
                            if ((Data_Values.Ch_1_AM_Sel) <  AM_Off)
                                Data_Values.Ch_1_AM_Sel =  AM_On;
                            Display_Value = Data_Values.Ch_1_AM_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                            Data_Values.Ch_1_AM_Sel = Data_Values.Ch_1_AM_Sel + 1;
                            if ((Data_Values.Ch_1_AM_Sel) >  AM_On)
                                Data_Values.Ch_1_AM_Sel =  AM_Off;
                            Display_Value = Data_Values.Ch_1_AM_Sel;
			}
			else if Sel_Key
			{
                            if (Data_Values.Ch_1_AM_Sel == AM_Off)
                            {
                                Sel_Fn = Ch_1_FM;
                                HDWordWriteSPI(AM_Offset, Data_Values.Ch_1_AM_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_FM_Sel;
                            }
                            else
                            {
                                Sel_Fn = Ch_1_AM_Level;
                                HDWordWriteSPI(AM_Offset, Data_Values.Ch_1_AM_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_AM_Level_Sel;
                            }
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                        Ch1_AM_On = (Data_Values.Ch_1_AM_Sel == AM_On); /* make these boolean */
		}

                /*************************************************************************/
		/* Currently in Ch_2_AM Function - act on legal button presses           */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_AM)
		{
			/* Down */
			if Down_Keys
			{
                            Data_Values.Ch_2_AM_Sel = Data_Values.Ch_2_AM_Sel - 1;
                            if ((Data_Values.Ch_2_AM_Sel) <  AM_Off)
                                Data_Values.Ch_2_AM_Sel =  AM_On;
                            Display_Value = Data_Values.Ch_2_AM_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                            Data_Values.Ch_2_AM_Sel = Data_Values.Ch_2_AM_Sel + 1;
                            if ((Data_Values.Ch_2_AM_Sel) >  AM_On)
                                Data_Values.Ch_2_AM_Sel =  AM_Off;
                            Display_Value = Data_Values.Ch_2_AM_Sel;
			}
			else if Sel_Key
			{
                            if (Data_Values.Ch_2_AM_Sel == AM_Off)
                            {
                                Sel_Fn = Ch_2_FM;
                                HDWordWriteSPI((Channel_Set_Array_Size + AM_Offset), Data_Values.Ch_2_AM_Sel);
				Display_Value = Data_Values.Ch_2_FM_Sel;
                            }
                            else
                            {
                                Sel_Fn = Ch_2_AM_Level;
                                HDWordWriteSPI((Channel_Set_Array_Size + AM_Offset), Data_Values.Ch_2_AM_Sel);
				Display_Value = Data_Values.Ch_2_AM_Level_Sel;
                            }
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                        Ch2_AM_On = (Data_Values.Ch_2_AM_Sel == AM_On); /* make these boolean */
		}


                /*************************************************************************/
		/* Currently in Ch_1_AM_Level Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_AM_Level)
		{
			/* Down */
			if Up_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_1_AM_Level_Sel = Data_Values.Ch_1_AM_Level_Sel + 1;
                                else
                                    Data_Values.Ch_1_AM_Level_Sel = Data_Values.Ch_1_AM_Level_Sel + 10;
				if ((Data_Values.Ch_1_AM_Level_Sel) >  (AM_Depth_Max ))
                                    Data_Values.Ch_1_AM_Level_Sel =  AM_Depth_Max;
				Display_Value = Data_Values.Ch_1_AM_Level_Sel;
			}
			/* Up */
			else if Down_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_1_AM_Level_Sel = Data_Values.Ch_1_AM_Level_Sel - 1;
                                else
                                    Data_Values.Ch_1_AM_Level_Sel = Data_Values.Ch_1_AM_Level_Sel - 10;
				if ((Data_Values.Ch_1_AM_Level_Sel) <  (AM_Depth_Min ))
                                    Data_Values.Ch_1_AM_Level_Sel =  AM_Depth_Min;
				Display_Value = Data_Values.Ch_1_AM_Level_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_AM_Freq;
                                HDWordWriteSPI(AM_Level_Offset, Data_Values.Ch_1_AM_Level_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_AM_Freq_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                        Ch1_AM_Mult = (long) ( (long long)Data_Values.Ch_1_AM_Level_Sel * ((long long)0x7fffffff/100));
		}

                /*************************************************************************/
		/* Currently in Ch_2_AM_Level Function - act on legal button presses     */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_AM_Level)
		{
			/* Down */
			if Up_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_2_AM_Level_Sel = Data_Values.Ch_2_AM_Level_Sel + 1;
                                else
                                    Data_Values.Ch_2_AM_Level_Sel = Data_Values.Ch_2_AM_Level_Sel + 10;
				if ((Data_Values.Ch_2_AM_Level_Sel) >  (AM_Depth_Max ))
                                    Data_Values.Ch_2_AM_Level_Sel =  AM_Depth_Max;
				Display_Value = Data_Values.Ch_2_AM_Level_Sel;
			}
			/* Up */
			else if Down_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_2_AM_Level_Sel = Data_Values.Ch_2_AM_Level_Sel - 1;
                                else
                                    Data_Values.Ch_2_AM_Level_Sel = Data_Values.Ch_2_AM_Level_Sel - 10;
				if ((Data_Values.Ch_2_AM_Level_Sel) <  (AM_Depth_Min ))
                                    Data_Values.Ch_2_AM_Level_Sel =  AM_Depth_Min;
				Display_Value = Data_Values.Ch_2_AM_Level_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_2_AM_Freq;
                                HDWordWriteSPI((Channel_Set_Array_Size + AM_Level_Offset), Data_Values.Ch_2_AM_Level_Sel);
				Display_Value = Data_Values.Ch_2_AM_Freq_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                        Ch2_AM_Mult = (long) ( (long long)Data_Values.Ch_2_AM_Level_Sel * ((long long)0x7fffffff/100));
		}


                /*************************************************************************/
		/* Currently in Ch_1_AM_Freq Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_AM_Freq)
		{
			/* Up */
			if Up_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_1_AM_Freq_Sel = Data_Values.Ch_1_AM_Freq_Sel + 1;
                                else
                                    Data_Values.Ch_1_AM_Freq_Sel = Data_Values.Ch_1_AM_Freq_Sel + 20;
				if ((Data_Values.Ch_1_AM_Freq_Sel) >  (AM_Freq_Max ))
                                    Data_Values.Ch_1_AM_Freq_Sel =  AM_Freq_Max;
				Display_Value = Data_Values.Ch_1_AM_Freq_Sel;
			}
			/* Down */
			else if Down_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_1_AM_Freq_Sel = Data_Values.Ch_1_AM_Freq_Sel - 1;
                                else
                                    Data_Values.Ch_1_AM_Freq_Sel = Data_Values.Ch_1_AM_Freq_Sel - 20;
				if ((Data_Values.Ch_1_AM_Freq_Sel) <  (AM_Freq_Min ))
                                    Data_Values.Ch_1_AM_Freq_Sel =  AM_Freq_Min;
				Display_Value = Data_Values.Ch_1_AM_Freq_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_FM;
                                HDWordWriteSPI(AM_Freq_Offset, Data_Values.Ch_1_AM_Freq_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_FM_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                 	/* calculate the phase increment for Ch1 AM*/
                        Temp_Calc_Store = 0x100000000; // 2^32
                        Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_1_AM_Freq_Sel;
                        Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz
                        Ch1_AM_Phase_Inc = (long) Temp_Calc_Store;
		}

                /*************************************************************************/
		/* Currently in Ch_2_AM_Freq Function - act on legal button presses      */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_AM_Freq)
		{
			/* Up */
			if Up_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_2_AM_Freq_Sel = Data_Values.Ch_2_AM_Freq_Sel + 1;
                                else
                                    Data_Values.Ch_2_AM_Freq_Sel = Data_Values.Ch_2_AM_Freq_Sel + 20;
				if ((Data_Values.Ch_2_AM_Freq_Sel) >  (AM_Freq_Max ))
                                    Data_Values.Ch_2_AM_Freq_Sel =  AM_Freq_Max;
				Display_Value = Data_Values.Ch_2_AM_Freq_Sel;
			}
			/* Down */
			else if Down_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_2_AM_Freq_Sel = Data_Values.Ch_2_AM_Freq_Sel - 1;
                                else
                                    Data_Values.Ch_2_AM_Freq_Sel = Data_Values.Ch_2_AM_Freq_Sel - 20;
				if ((Data_Values.Ch_2_AM_Freq_Sel) <  (AM_Freq_Min ))
                                    Data_Values.Ch_2_AM_Freq_Sel =  AM_Freq_Min;
				Display_Value = Data_Values.Ch_2_AM_Freq_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_2_FM;
                                HDWordWriteSPI((Channel_Set_Array_Size + AM_Freq_Offset), Data_Values.Ch_2_AM_Freq_Sel);
				Display_Value = Data_Values.Ch_2_FM_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                        /* calculate the phase increment for Ch1 AM*/
                        Temp_Calc_Store = 0x100000000; // 2^32
                        Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_2_AM_Freq_Sel;
                        Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz
                        Ch2_AM_Phase_Inc = (long) Temp_Calc_Store;
		}


                /*************************************************************************/
		/* Currently in Ch_1_FM Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_FM)
		{
			/* Down */
			if Down_Keys
			{
                            Data_Values.Ch_1_FM_Sel = Data_Values.Ch_1_FM_Sel - 1;
                            if ((Data_Values.Ch_1_FM_Sel) <  FM_Off)
                                Data_Values.Ch_1_FM_Sel =  FM_On;
                            Display_Value = Data_Values.Ch_1_FM_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                            Data_Values.Ch_1_FM_Sel = Data_Values.Ch_1_FM_Sel + 1;
                            if ((Data_Values.Ch_1_FM_Sel) >  FM_On)
                                Data_Values.Ch_1_FM_Sel =  FM_Off;
                            Display_Value = Data_Values.Ch_1_FM_Sel;
			}
			else if Sel_Key
			{
                            if (Data_Values.Ch_1_FM_Sel == FM_Off)
                            {
                                Sel_Fn = Ch_1_Freq;
                                HDWordWriteSPI(FM_Offset, Data_Values.Ch_1_FM_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Freq_Sel;
                            }
                            else
                            {
                                Sel_Fn = Ch_1_FM_Excursion;
				Display_Value = Data_Values.Ch_1_FM_Excursion_Sel;
                            }
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
                                HDWordWriteSPI(FM_Offset, Data_Values.Ch_1_FM_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Mode;
			}
			else;
                        /* set flag */
                        Ch1_FM_On =  (Data_Values.Ch_1_FM_Sel == FM_On); /* make these boolean */
		}

                /*************************************************************************/
		/* Currently in Ch_2_FM Function - act on legal button presses           */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_FM)
		{
			/* Down */
			if Down_Keys
			{
                            Data_Values.Ch_2_FM_Sel = Data_Values.Ch_2_FM_Sel - 1;
                            if ((Data_Values.Ch_2_FM_Sel) <  FM_Off)
                                Data_Values.Ch_2_FM_Sel =  FM_On;
                            Display_Value = Data_Values.Ch_2_FM_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                            Data_Values.Ch_2_FM_Sel = Data_Values.Ch_2_FM_Sel + 1;
                            if ((Data_Values.Ch_2_FM_Sel) >  FM_On)
                                Data_Values.Ch_2_FM_Sel =  FM_Off;
                            Display_Value = Data_Values.Ch_2_FM_Sel;
			}
			else if Sel_Key
			{
                            if (Data_Values.Ch_2_FM_Sel == FM_Off)
                            {
                                Sel_Fn = Ch_2_Freq;
                                HDWordWriteSPI((Channel_Set_Array_Size + FM_Offset), Data_Values.Ch_2_FM_Sel);
				Display_Value = Data_Values.Ch_2_Freq_Sel;
                            }
                            else
                            {
                                Sel_Fn = Ch_2_FM_Excursion;
                                HDWordWriteSPI((Channel_Set_Array_Size + FM_Offset), Data_Values.Ch_2_FM_Sel);
				Display_Value = Data_Values.Ch_2_FM_Excursion_Sel;
                            }
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                        /* set flag */
                        Ch2_FM_On =  (Data_Values.Ch_2_FM_Sel == FM_On); /* make these boolean */
		}


                /*************************************************************************/
		/* Currently in Ch_1_FM_Excursion Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_FM_Excursion)
		{
			/* Down */
			if Down_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_1_FM_Excursion_Sel = Data_Values.Ch_1_FM_Excursion_Sel - 1;
                                else
                                    Data_Values.Ch_1_FM_Excursion_Sel = Data_Values.Ch_1_FM_Excursion_Sel - 20;
				if ((Data_Values.Ch_1_FM_Excursion_Sel) <  (FM_Excursion_Min))
                                    Data_Values.Ch_1_FM_Excursion_Sel =  FM_Excursion_Min;
				Display_Value = Data_Values.Ch_1_FM_Excursion_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_1_FM_Excursion_Sel = Data_Values.Ch_1_FM_Excursion_Sel + 1;
                                else
                                    Data_Values.Ch_1_FM_Excursion_Sel = Data_Values.Ch_1_FM_Excursion_Sel + 20;
				if ((Data_Values.Ch_1_FM_Excursion_Sel) >  (FM_Excursion_Max))
                                    Data_Values.Ch_1_FM_Excursion_Sel =  FM_Excursion_Max;
				Display_Value = Data_Values.Ch_1_FM_Excursion_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_FM_Freq;
                                HDWordWriteSPI(FM_Excursion_Offset, Data_Values.Ch_1_FM_Excursion_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_FM_Freq_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
                        else;
                        /* calculate and load CH1 FM table here */
                        Temp_Calc_Store = 0x100000000; // 2^32
                        Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_1_FM_Excursion_Sel;
                        Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz - this is the peak
                        for(temp = 0; temp < 2048; temp++)
                        {
                            /* watch out - you are hard coding for the scale different between the SINE lut and FM LUT*/
                            Ch1_FM_LUT[temp] = (long)(((long long) Temp_Calc_Store * (long long) Sine_LUT[temp*2]) / 0x1000000);
                        }
		}

                /*************************************************************************/
		/* Currently in Ch_2_FM_Excursion Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_FM_Excursion)
		{
			/* Down */
			if Down_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_2_FM_Excursion_Sel = Data_Values.Ch_2_FM_Excursion_Sel - 1;
                                else
                                    Data_Values.Ch_2_FM_Excursion_Sel = Data_Values.Ch_2_FM_Excursion_Sel - 20;
				if ((Data_Values.Ch_2_FM_Excursion_Sel) <  (FM_Excursion_Min))
                                    Data_Values.Ch_2_FM_Excursion_Sel =  FM_Excursion_Min;
				Display_Value = Data_Values.Ch_2_FM_Excursion_Sel;
			}
			/* Up */
			else if Up_Keys
			{
                                if(Fast_Change == 1)
                                    Data_Values.Ch_2_FM_Excursion_Sel = Data_Values.Ch_2_FM_Excursion_Sel + 1;
                                else
                                    Data_Values.Ch_2_FM_Excursion_Sel = Data_Values.Ch_2_FM_Excursion_Sel + 20;
				if ((Data_Values.Ch_2_FM_Excursion_Sel) >  (FM_Excursion_Max))
                                    Data_Values.Ch_2_FM_Excursion_Sel =  FM_Excursion_Max;
				Display_Value = Data_Values.Ch_2_FM_Excursion_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_2_FM_Freq;
                                HDWordWriteSPI((Channel_Set_Array_Size + FM_Excursion_Offset), Data_Values.Ch_2_FM_Excursion_Sel);
				Display_Value = Data_Values.Ch_2_FM_Freq_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                        /* calculate and load CH1 FM table here */
                        Temp_Calc_Store = 0x100000000; // 2^32
                        Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_2_FM_Excursion_Sel;
                        Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz - this is the peak
                        for(temp = 0; temp < 2048; temp++)
                        {
                            /* watch out - you are hard coding for the scale different between the SINE lut and FM LUT*/
                            Ch2_FM_LUT[temp] = (long)(((long long) Temp_Calc_Store * (long long) Sine_LUT[temp*2]) / 0x1000000);
                        }
		}


                /*************************************************************************/
		/* Currently in Ch_1_FM_Freq Function - act on legal button presses   */
		/*************************************************************************/
                else if (Sel_Fn == Ch_1_FM_Freq)
		{
			/* Down */
			if Down_Keys
			{
				if (Data_Values.Ch_1_FM_Freq_Sel >=  (FM_Freq_Min + Fast_Change))
				{
					Data_Values.Ch_1_FM_Freq_Sel = Data_Values.Ch_1_FM_Freq_Sel - Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_1_FM_Freq_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Ch_1_FM_Freq_Sel <= (FM_Freq_Max - Fast_Change))
				{
					Data_Values.Ch_1_FM_Freq_Sel = Data_Values.Ch_1_FM_Freq_Sel + Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_1_FM_Freq_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_Freq;
                                HDWordWriteSPI(FM_Freq_Offset, Data_Values.Ch_1_FM_Freq_Sel); /* Write Frequency Offset to EEPROM */
				Display_Value = Data_Values.Ch_1_Freq_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                        /* calculate the phase increment for Ch1 AM*/
                        Temp_Calc_Store = 0x100000000; // 2^32
                        Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_1_FM_Freq_Sel;
                        Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz
                        Ch1_FM_Phase_Inc = (long) Temp_Calc_Store;
		}

                /*************************************************************************/
		/* Currently in Ch_2_FM_Freq Function - act on legal button presses      */
		/*************************************************************************/
                else if (Sel_Fn == Ch_2_FM_Freq)
		{
			/* Down */
			if Down_Keys
			{
				if (Data_Values.Ch_2_FM_Freq_Sel >=  (FM_Freq_Min + Fast_Change))
				{
					Data_Values.Ch_2_FM_Freq_Sel = Data_Values.Ch_2_FM_Freq_Sel - Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_2_FM_Freq_Sel;
			}
			/* Up */
			else if Up_Keys
			{
				if (Data_Values.Ch_2_FM_Freq_Sel <= (FM_Freq_Max - Fast_Change))
				{
					Data_Values.Ch_2_FM_Freq_Sel = Data_Values.Ch_2_FM_Freq_Sel + Fast_Change;
                                        /* You might need to do somehting in here..... */
 				}
				Display_Value = Data_Values.Ch_2_FM_Freq_Sel;
			}
			else if Sel_Key
			{
                                Sel_Fn = Ch_1_Freq;
                                HDWordWriteSPI((Channel_Set_Array_Size + FM_Freq_Offset), Data_Values.Ch_2_FM_Freq_Sel);
				Display_Value = Data_Values.Ch_2_Freq_Sel;
			}
			else if (Keypress_Read == Exit)
			{
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			else;
                        /* calculate the phase increment for Ch1 AM*/
                        Temp_Calc_Store = 0x100000000; // 2^32
                        Temp_Calc_Store = Temp_Calc_Store * Data_Values.Ch_2_FM_Freq_Sel;
                        Temp_Calc_Store = Temp_Calc_Store / (Sample_Rate);  // freq is in Hz
                        Ch2_FM_Phase_Inc = (long) Temp_Calc_Store;
		}


                /************************************************************************/
		/* Currently in SAVE Function, act on legal button presses				*/
		/************************************************************************/
		else if(Sel_Fn == Save_Sel_Fn)
		{
			/*		This section miplements the save function 					*/
			/* Up 																*/
			if Up_Keys
			{
				Memory_Bank += 1;
				if (Memory_Bank > Max_Mem_Banks_With_Erase)
					Memory_Bank = Default_Mem_Bank;
				Display_Value = Memory_Bank;
			}
			/* Down */
			else if Down_Keys
			{
				if (Memory_Bank == Default_Mem_Bank)
					Memory_Bank = Max_Mem_Banks;
				else
					Memory_Bank -= 1;
				Display_Value = Memory_Bank;
			}
			/* Select Key hit */
                        /* write memory for normal save operations */
                        else if (Sel_Key && (Memory_Bank <= Max_Mem_Banks))
			{
			/********************************************************************/
			/*		Save the data                                       */
			/********************************************************************/
                                Channel_Mem_Offset =  Memory_Bank * ParmSet_Array_Size;
                                HDWordWriteSPI(Channel_Mem_Offset + Freq_Offset, Data_Values.Ch_1_Freq_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Level_Offset, Data_Values.Ch_1_Level_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Waveform_Offset, Data_Values.Ch_1_Waveform_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Ratio_Offset, Data_Values.Ch_1_Ratio_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Stop_Freq_Offset, Data_Values.Ch_1_Sweep_Stop_Freq_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Type_Offset, Data_Values.Ch_1_Sweep_Type_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Time_Offset, Data_Values.Ch_1_Sweep_Time_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Phase_Offset, Data_Values.Ch_1_Phase_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Offset, Data_Values.Ch_1_AM_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Offset, Data_Values.Ch_1_FM_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Pulse_Offset, Data_Values.Ch_1_Pulse_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Level_Offset, Data_Values.Ch_1_AM_Level_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Freq_Offset, Data_Values.Ch_1_AM_Freq_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Freq_Offset, Data_Values.Ch_1_FM_Freq_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Excursion_Offset, Data_Values.Ch_1_FM_Excursion_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Cycles_On_Offset, Data_Values.Ch_1_PM_Cycles_On_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Cycles_Off_Offset, Data_Values.Ch_1_PM_Cycles_Off_Sel);

                                Channel_Mem_Offset = Channel_Set_Array_Size + Memory_Bank * ParmSet_Array_Size;
                                HDWordWriteSPI(Channel_Mem_Offset + Freq_Offset, Data_Values.Ch_2_Freq_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Level_Offset, Data_Values.Ch_2_Level_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Waveform_Offset, Data_Values.Ch_2_Waveform_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Ratio_Offset, Data_Values.Ch_2_Ratio_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Stop_Freq_Offset, Data_Values.Ch_2_Sweep_Stop_Freq_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Type_Offset, Data_Values.Ch_2_Sweep_Type_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Time_Offset, Data_Values.Ch_2_Sweep_Time_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Phase_Offset, Data_Values.Ch_2_Phase_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Offset, Data_Values.Ch_2_AM_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Offset, Data_Values.Ch_2_FM_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Pulse_Offset, Data_Values.Ch_2_Pulse_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Level_Offset, Data_Values.Ch_2_AM_Level_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Freq_Offset, Data_Values.Ch_2_AM_Freq_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Freq_Offset, Data_Values.Ch_2_FM_Freq_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Excursion_Offset, Data_Values.Ch_2_FM_Excursion_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Cycles_On_Offset, Data_Values.Ch_2_PM_Cycles_On_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Cycles_Off_Offset, Data_Values.Ch_2_PM_Cycles_Off_Sel);

				Sel_Fn = Choosing_Fn;
				Mode = Ch_1_Mode;
				Display_Value = Mode;
			}
                        /* Exit */
			else if (Sel_Key && (Memory_Bank == Mem_Banks_Do_Not_Erase))
			{
                                Memory_Bank = Default_Mem_Bank;
                                Step_Size = 0;
				Sel_Fn = Choosing_Fn;
				Mode = Ch_1_Mode;
				Display_Value = Mode;
			}
                        else if (Sel_Key && (Memory_Bank == Mem_Banks_Erase))
			{
                            for (Memory_Bank = Default_Mem_Bank; Memory_Bank <=  Max_Mem_Banks; Memory_Bank++)
                            {
				/********************************************************************/
                                /* Set about resetting memory to defaults                           */
                                /* Refer to header file for default values...                       */
				/********************************************************************/
                                Channel_Mem_Offset =  Memory_Bank * ParmSet_Array_Size;
                                HDWordWriteSPI(Channel_Mem_Offset + Freq_Offset, Freq_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Level_Offset, Level_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Waveform_Offset, Sine_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Ratio_Offset, Ratio_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Stop_Freq_Offset, Sweep_Stop_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Type_Offset, Sweep_Type_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Time_Offset, Sweep_Time_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Phase_Offset, Phase_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Offset, AM_Depth_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Offset, FM_On_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Pulse_Offset, Pulse_On_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Level_Offset, AM_Depth_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Freq_Offset, AM_Freq_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Freq_Offset, FM_Freq_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Excursion_Offset, FM_Excursion_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Cycles_On_Offset, PM_Cycles_On_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Cycles_Off_Offset, PM_Cycles_Off_Default);

                                Channel_Mem_Offset = Channel_Set_Array_Size + Memory_Bank * ParmSet_Array_Size;
                                HDWordWriteSPI(Channel_Mem_Offset + Freq_Offset, Freq_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Level_Offset, Level_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Waveform_Offset, Sine_Sel);
                                HDWordWriteSPI(Channel_Mem_Offset + Ratio_Offset, Ratio_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Stop_Freq_Offset, Sweep_Stop_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Type_Offset, Sweep_Type_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Sweep_Time_Offset, Sweep_Time_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Phase_Offset, Phase_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Offset, AM_Depth_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Offset, FM_On_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Pulse_Offset, Pulse_On_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Level_Offset, AM_Depth_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + AM_Freq_Offset, AM_Freq_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Freq_Offset, FM_Freq_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + FM_Excursion_Offset, FM_Excursion_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Cycles_On_Offset, PM_Cycles_On_Default);
                                HDWordWriteSPI(Channel_Mem_Offset + Cycles_Off_Offset, PM_Cycles_Off_Default);
                            }
                            Memory_Bank = Default_Mem_Bank;
                            Sel_Fn = Choosing_Fn;
                            Mode = Ch_1_Mode;
                            Display_Value = Mode;
			}


			/* Exit */
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = Choosing_Fn;
				Mode = Ch_1_Mode;
				Display_Value = Mode;
			}
			else;
		}

		/************************************************************************/
		/* Currently in LOAD Function, act on legal button presses 				*/
		/************************************************************************/
		else if(Sel_Fn == Load_Sel_Fn)
		{
			/********************************************************************/
			/*		This section miplements the load data function 				*/
			/********************************************************************/
			/* Up */
			if Up_Keys
			{
				Memory_Bank += 1;
				if (Memory_Bank > Max_Mem_Banks)
					Memory_Bank = 0;
				Display_Value = Memory_Bank;
			}
			/* Down */
			else if Down_Keys
			{
				if (Memory_Bank == 0)
					Memory_Bank = Max_Mem_Banks;
				else
					Memory_Bank -= 1;
				Display_Value = Memory_Bank;
			}
			/* Select Key hit */
			else if Sel_Key
			{
				/********************************************************************/
				/*		Load the data 												*/
				/********************************************************************/		
				Load_From_Memory(Memory_Bank); /* Read Data from ROM */
				/* now program the values the user loaded...*/
				Update_Atten_Set(Mute);

				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}

		/************************************************************************/
		/* Exit                                                                 */
		/************************************************************************/
			else if Exit_Key
			{
				Step_Size = 0;
				Sel_Fn = Choosing_Fn;
				Display_Value = Mode;
			}
			
			else
				Update_Display = 0;
		}
	
		if (Update_Display)	
			Display_Data(Sel_Fn, Display_Value);
		else;


	}  /* Done changing values for the user */


	/* Clears the flag saying that the push button press has been addressed */
	Step_Size = 0;

	Auto_Revert = Auto_Revert - 1;   /* If no key pressed, then decrement this counter */
	if 	(Auto_Revert <= 0)   	/* Then force the system to volume mode */
		{
                        /* If key pressed, reset the counter */
                        Auto_Revert = Auto_Revert_Init_Val;
			Band = 0;
                        Sel_Fn = Choosing_Fn;
			Display_Value = Mode;
			Display_Data(Sel_Fn, Display_Value);
		}

        /* delay in main loop to allow the HMI to work at a sane speed */
        DelayUs(Key_Press_Delay_Us);

	}
/* and just stay in this loop till forever...*/
while (1);
}
