/* 28/6/07                                                  */
/* End of build 1 of major rewrite to implement rotary ctrl */
/* basic functionality for:
	- Load and save three parm sets
	- Rotary control basics
	- Memory map for load save
	- menu structure for rotary control
*/
/* 28/10/2013                                               */
/* Added a bunch of stuff for PIC32MX450F256H               */
/* includes significant update to use generic defines       */
/* and reduces processor dependance of code used            */
/* 24/12/2013                                               */
/* Generally all working - bar tweaks                       */
/*                                                          */

// Let compile time pre-processor calculate the PR1 (period)
#define SYS_FREQ             (80000000L)
#define PB_DIV                 1
#define PRESCALE               256
#define TOGGLES_PER_SEC        1
#define T1_TICK               (SYS_FREQ/PB_DIV/PRESCALE/TOGGLES_PER_SEC)

//Apears this version of the compiler omits this definition
// if you get compiler errors, then these have probably turned back up :)
#define IN_FN_PPS2_RPD3 RPD3Rbits.RPD3R
#define IN_PIN_PPS2_SDI1 0
#define IN_FN_PPS2_RPG7 RPG7Rbits.RPG7R
#define IN_PIN_PPS2_SDI2 6

/* RB0 - spare (output)*/
/* RB1 - spare (output)*/
/* RB2 - spare (output)*/
/* RB3 - spare (output)*/
/* RB4 - spare (output)*/
/* RB5 - spare (output)*/
/* RB6 - Rotary_Encoder_Select_0 (input)*/
#define Rotary_Encoder_Select_0 64
/* RB7 - Rotary_Encoder_Select_1 (input)*/
#define Rotary_Encoder_Select_1 128

/* Old Rotary Encoder = 00XXXXXX*/
#define Old_Rotary_Encoder 0x0000
/* Type A rotary Encoder = 10XXXXXX*/
#define Type_A_Rotary_Encoder 0x0080
/* Type B Rotary Encoder = 01XXXXXX*/
#define Type_B_Rotary_Encoder 0x0040

/* RB8 - LCD0 (output) */
/* RB9 - LCD1 (output) */
/* RB10 - LCD2 (output) */
/* RB11 - LCD3 (output) */
/* RB12 - LCD4 (output) */
/* RB13 - LCD5 (output) */
/* RB14 - LCD6 (output) */
/* RB15 - LCD7 (output) */
#define Clear_LCD 1
#define Home_LCD 2
#define No_Blink_LCD 8
#define Blink_LCD 14

/* Function Select 							*/
/* 	RS	RW 	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	1	1	1	0	0	0	*/
#define LCD_Function 0x38

/* Display On */
/* 	RS	RW 	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	0	0	1	1	1	1	*/
#define LCD_Display_On 0x0F

/* Display Clear */
/* 	RS	RW 	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	0	0	0	0	0	1	*/
#define LCD_Display_Clear 0x01

/* Entry Mode */
/* 	RS	RW 	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	0	0	0	0	0	1	1	1	*/
#define LCD_Display_Entry_Mode 0x06

/* DDRAM Line 1 Address */
/* 	RS(d6)	RW(d5) 	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	1	0	0	0	0	0	0	0	*/

#define LCD_Line_1 0x80

/* DDRAM Line 2 Address */
/* 	RS(d6)	RW(d5) 	E(d7)	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	0	0	1	1	0	1       0	0	0	0	0	*/

#define LCD_Line_2      0xC0

/* Enable Write Data to LCD */
/* 	RS(d6)	RW(d5) 	E(d7)	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	1	0	0	d7	d6	d5	d4	d3	d2	d1	d0	*/
#define Write_Data_Enable_LCD 64

/* Write Data to LCD */
/* 	RS(d6)	RW(d5) 	E(d7)	DB7     DB6	DB5	DB4	DB3	DB2	DB1	DB0 */
/*	1	0	1	d7	d6	d5	d4	d3	d2	d1	d0	*/
#define Write_Data_LCD 192
#define Write_Control_LCD 32
#define Write_Control_Enable_LCD 128
#define	LCD_Data(x)	{ int _dt; \
                          DelayUs(1);\
			  _dt = x<<8; \
                          _dt = _dt + (PORTB & 0x00FF);\
                          PORTB = _dt;\
                          DelayUs(1);}

#define Write_Control_LCD_CSEL DelayUs(1); PORTD = PORTD & (0xFFFF - (Write_Data_Enable_LCD + Write_Control_Enable_LCD + Write_Control_LCD)); DelayUs(1)
#define Write_Control_Enable_LCD_CSEL DelayUs(1); PORTD = PORTD | Write_Control_Enable_LCD; DelayUs(1)
#define Write_Data_LCD_CSEL DelayUs(1); PORTD = PORTD | (Write_Control_Enable_LCD + Write_Data_Enable_LCD); DelayUs(1); PORTD = PORTD & (0xFFFF - Write_Control_LCD);DelayUs(50)
#define Write_Data_Enable_LCD_CSEL DelayUs(1); PORTD = PORTD | Write_Data_Enable_LCD; DelayUs(1); PORTD = PORTD & (0xFFFF - Write_Control_Enable_LCD - Write_Control_LCD); DelayUs(50)
// LCD wants 37uS delay after write
#define LCD_Delay_After_Write 40
// LCD Data Setting Time - use 5uS
#define LCD_Settle_Before_Write 5
// LCD Control 1.5mS Setting Time
#define LCD_Settle_After_Control 2


/* port C is oscillators */

/* RD0 SPARE */
/* RD1 = SPI_CS1*/
#define CSel 0x002
/* RD2 = SPI_SCK1*/
/* RD3 = SPI_SDI1 = "0000" = input*/
#define SDI_Out 16
/* RD4 = SDI_SDO1 = "1000"*/
/* RD5 = LCD_RW (output)*/
#define R_W_LCD 32
/* RD6 = LCD_RS (output)*/
#define RS_LCD 64
/* RD7 = LCD_CS (output)*/
#define Enable_LCD 128

/* RD8 = DSP_RESET*/
#define Reset_DSP       0x0100   /* Bit 8 of port D */
/* RD9 = SPI_SS1 - "0111"*/
/* RD10 = RESET_DAC*/
#define Reset_DAC 	0x0400   /* bit 10 of port D */
/* RD11 = UNUSED (output)*/
/*
 * Drive LCD Control for dispaly select using port B Output
 */

/* RE0 = SPARE*/
/* RE1 = MODE_SEL (input)*/
/* RE2 = FN_SEL (input)*/
/* RE3 = PHASEA (input)*/
/* RE4 = PHASEB (input)*/
/* RE5 = SPARE*/
/* RE6 = SPARE*/
/* RE7 = Select_PGA2320*/
#define PGA2320_Select 0x0080

#define Port_E_Button_Pressed   (2+4+8+16)

/* RF0 EEPROM Chip Select_Active Low*/
#define EEPROM_Select 1
/* RF1 EEPROM Chip Hold_Active Low*/
#define EEPROM_Hold 2
/* RF1 SPARE (output)*/
/* RF4 UNUSED (output)*/
/* RF5 UNUSED (output)*/

/* RG6 = SPI_SCK2 */
/* RG7 = SPI_SDI2 "0001" */
/* RG8 = SPI_SDO2 = "0110" */
/* RG9 = SPI_SS2 = "0110" */

/*
PORT C definitions  - NEEDS FIXING
*/
#define Port_XXXX_Button_Pressed   3


/* Input port bits */
/* Input Port E */
#define Band_Select         1
#define Function_Select     2
#define Max_Bands           3


/* Display Strings */
#define Blank_Char_Init 	0x20
#define Line1_Init      	"  TGM Was Here  "
#define Line2_Init     	 	"DDS V1.1  Dec 13"
#define Line2_Select_Init	"Press Select:   "
#define Line1_Channel_1_Init   	"Channel 1 Setup:"
#define Line1_Channel_2_Init   	"Channel 2 Setup:"
#define Line1_Save_Init 	"Save to bank:   "
#define Line1_Erase_Check 	"Erase Memory?   "
#define Line2_Erase_Check_Y 	"Restore Defaults"
#define Line2_Erase_Check_N 	"NO! Let me out! "
#define Line1_Load_Init 	"Load from bank: "
#define Line1_Level_Init  	"Attenuation:  dB"
#define Line2_Level_Init  	"Ref Max Output  "
#define Boot_Msg 666
#define Line1_Choose_Str_Init	"Select Function:"

/*
Stuff to allow modes to be tracked
Use these to control the state machine
*/
#define Choosing_Fn	0
#define Ch_1_Mode       1
#define Ch_2_Mode 	2
#define Save_Mode	3
#define Load_Mode	4

/*
Stuff to allow functions to be tracked in Channel 1 mode
*/
#define Ch_1_Freq       10
#define Ch_1_Level      11
#define Ch_1_Waveform   12
#define Ch_1_Sweep      13
#define Ch_1_Phase      14
#define Ch_1_AM         15
#define Ch_1_FM         16
#define Ch_1_Pulse      17

/*
Stuff to allow functions to be tracked in Channel 2 mode
*/
#define Ch_2_Freq       20
#define Ch_2_Level      21
#define Ch_2_Waveform   22
#define Ch_2_Sweep      23
#define Ch_2_Phase      24
#define Ch_2_AM         25
#define Ch_2_FM         26
#define Ch_2_Pulse      27

/*
Stuff to allow functions to be tracked in Waveform Select mode
*/
#define Sine_Sel        120
#define Triangle_Sel    121
#define Square_Sel      122
#define Pulse_Sel       123
#define Arb_Sel         124

/*
Stuff to allow functions to be tracked in Sweep Select mode
*/
#define Sweep_Off       130
#define Sweep_Lin       131
#define Ch_1_Sweep_Stop      135
#define Ch_2_Sweep_Stop      235
#define Ch_1_Sweep_Time      137
#define Ch_2_Sweep_Time      237

/*
Stuff to allow functions to be tracked in AM Select mode
*/
#define AM_Off          150
#define AM_On           151
#define Ch_1_AM_Level   155
#define Ch_1_AM_Freq    156
#define Ch_2_AM_Level   255
#define Ch_2_AM_Freq    256

/* Define a state for user to set frequency step sizes*/
#define Freq_Speed_Counter_1_fn 190
#define Freq_Speed_Counter_2_fn 191

/*
Stuff to allow functions to be tracked in FM Select mode
*/
#define FM_Off          160
#define FM_On           161
#define Ch_1_FM_Excursion    165
#define Ch_1_FM_Freq         166
#define Ch_2_FM_Excursion    265
#define Ch_2_FM_Freq         266

/*
Stuff to allow functions to be tracked in Pulse Select mode
*/
#define Pulse_Off       170
#define Pulse_On        171
#define Ch_1_Ratio      178
#define Ch_1_PM_Cycles_On  175
#define Ch_1_PM_Cycles_Off 176
#define Ch_2_Ratio      278
#define Ch_2_PM_Cycles_On  275
#define Ch_2_PM_Cycles_Off 276

/*
Stuff to allow functions to be tracked in Save mode
*/
#define Save_Sel_Fn 	30

/*
Stuff to allow functions to be tracked in Load mode
*/
#define Load_Sel_Fn 	40

/*
Stuff to allow memory bank in use to be tracked
*/
#define Mem_Banks_Do_Not_Erase 3
#define Mem_Banks_Erase 4
#define Max_Mem_Banks_With_Erase 4
#define Max_Mem_Banks 	2
#define Default_Mem_Bank 0

/*
more stuff 
Use this bitfield to watch what buttons are happening...
*/
#define  HMI_Port PORTE
//#define  Vol_Dn (unsigned char) 1
//#define  Vol_Up (unsigned char) 2
//#define  Vals_2 (unsigned char) 32
//#define  Mode_Sel (unsigned char) 64
#define  Exit (unsigned char) 2     //pretty sure this is bit 1 of port E
#define  Sel (unsigned char) 4      //pretty sure this is bit 2 of port E
#define  Vals_0 (unsigned char) 8   //pretty sure this is bit 3 of port E
#define  Vals_1 (unsigned char) 16  //pretty sure this is bit 4 of port E
#define Key_Press_Delay_Us 150
#define Slow_Counter_Init 1500
#define Fast_Counter_Init 80

#define Phase_0 (unsigned char) 0
#define Phase_1 Vals_0
#define Phase_2 Vals_0 + Vals_1
#define Phase_3 Vals_1

#define Rot_Up Vals_1
#define Rot_Down Vals_0 + Vals_1

#define Up_Keys (Keypress_Read == Vals_1)
#define Down_Keys (Keypress_Read == (Vals_0 + Vals_1))
#define Sel_Key (Keypress_Read == Sel)
#define Exit_Key (Keypress_Read == Exit)

/*Define Value for Mute Volume */
#define  Mute 255

/* Auto Repeat Counts - how long to wait before starting to Auto Repeat*/
#define Auto_Repeat_Count 6


typedef struct{
		int Ch_1_Freq_Sel;   /* Freq is in 10th of Hz*/
                int Ch_1_Level_Sel;  /* Level is attenuation 0 - 256 by 0.5dB*/
                int Ch_1_Waveform_Sel;
                int Ch_1_Ratio_Sel;  /* Ratio is 0-360 degrees in 2^16 bits*/
                int Ch_1_Sweep_Stop_Freq_Sel;
                int Ch_1_Sweep_Type_Sel;
                int Ch_1_Sweep_Time_Sel; /* tenths of a second */
                int Ch_1_Phase_Sel;
                int Ch_1_AM_Sel;
                int Ch_1_FM_Sel;
                int Ch_1_Pulse_Sel;
                int Ch_1_AM_Level_Sel;
                int Ch_1_AM_Freq_Sel;
                int Ch_1_FM_Freq_Sel;
                int Ch_1_FM_Excursion_Sel;
                int Ch_1_PM_Cycles_On_Sel;
                int Ch_1_PM_Cycles_Off_Sel;
		int Ch_2_Freq_Sel;
                int Ch_2_Level_Sel;
                int Ch_2_Waveform_Sel;
                int Ch_2_Ratio_Sel;
                int Ch_2_Sweep_Stop_Freq_Sel;
                int Ch_2_Sweep_Type_Sel;
                int Ch_2_Sweep_Time_Sel; /* tenths of a second */
                int Ch_2_Phase_Sel;
                int Ch_2_AM_Sel;
                int Ch_2_FM_Sel;
                int Ch_2_Pulse_Sel;
                int Ch_2_AM_Level_Sel;
                int Ch_2_AM_Freq_Sel;
                int Ch_2_FM_Freq_Sel;
                int Ch_2_FM_Excursion_Sel;
                int Ch_2_PM_Cycles_On_Sel;
                int Ch_2_PM_Cycles_Off_Sel;
}  data_val_struct ;

#define Default_Mem_Bank 0

/* How big is each set of data in ROM?*/
#define ParmSet_Array_Size          256
#define Channel_Set_Array_Size      128

/* crossover and para data locations */
#define Freq_Offset            0
#define Level_Offset           4
#define Waveform_Offset        8
#define Ratio_Offset           12
#define Sweep_Stop_Freq_Offset 16
#define Sweep_Type_Offset      20
#define Phase_Offset           24
#define AM_Offset              28
#define FM_Offset              32
#define Pulse_Offset           36
#define AM_Level_Offset        40
#define AM_Freq_Offset         44
#define FM_Excursion_Offset    48
#define FM_Freq_Offset         52
#define Cycles_On_Offset       60
#define Cycles_Off_Offset      64
#define Sweep_Time_Offset      68


/*  Limit Values for Data Structures  */
#define	Freq_Default        1000
#define	Freq_Min            1
#define	Freq_Max            300000 /* This needs to be a bit less than fs/2 */
#define	Level_Default       0
#define	Level_Min           0
#define	Level_Max           60
#define	Waveform_Default    Sine_Sel
#define	Waveform_Min        120
#define	Waveform_Max        124
#define	Ratio_Default       50
#define	Ratio_Min           1
#define	Ratio_Max           99
#define	Sweep_Stop_Min      1           /* Hz */
#define	Sweep_Stop_Default  1000
#define	Sweep_Stop_Max      30000       /* Hz - again needs to be a touch under Fs/2 */
#define	Sweep_Time_Min      1
#define	Sweep_Time_Default  10
#define	Sweep_Time_Max      999 /* tenths of secs. dont know why I chose this - seems long enough - can be just about any value*/
#define	Sweep_Type_Default  Sweep_Off
#define	Sweep_Type_Min      Sweep_Off
#define	Sweep_Type_Max      Sweep_Lin
#define	Phase_Default       0
#define	Phase_Min           0
#define	Phase_Max           3600  /* tenths of a degree */
#define	AM_Depth_Default    0
#define	AM_Depth_Min        0   /* percent */
#define	AM_Depth_Max        100 /* percent */
#define	AM_Freq_Default     100 /* Hz */
#define	AM_Freq_Min         1   /* Hz */
#define	AM_Freq_Max         1000/* Hz */
#define	FM_Freq_Default     100 /* Hz */
#define	FM_Freq_Min         1   /* Hz */
#define	FM_Freq_Max         1000 /*  Hz */
#define	FM_Excursion_Default 100
#define	FM_Excursion_Min    1
#define	FM_Excursion_Max    1000 /* Hz */
#define	PM_Cycles_On_Default 3
#define	PM_Cycles_On_Min    1
#define	PM_Cycles_On_Max    1000
#define	PM_Cycles_Off_Default 10
#define	PM_Cycles_Off_Min   1
#define	PM_Cycles_Off_Max   1000
#define Pulse_On_Default    Pulse_Off
#define Pulse_On_Min        Pulse_Off
#define Pulse_On_Max        Pulse_On
#define AM_On_Default       AM_Off
#define AM_On_Min           AM_Off
#define AM_On_Max           AM_On
#define FM_On_Default       FM_Off
#define FM_On_Min           FM_Off
#define FM_On_Max           FM_On
#define Min_Freq_Steps      1
#define Max_Freq_Steps      1000


/* Sample rate of the DAC and thus the DDS timing cycle...*/
// TEST #define Sample_Rate 46875  //this is 3000000/64
#define Sample_Rate 62500  //this is 4000000/64
#define baud_rate (Sample_Rate * 64)
#define mclk (Sample_Rate * 256)
//Definitions - relate to Clock Rate and Sample Rate.
// TEST #define RODIV_PROG	4 - this gives you fs of about 46KHz
#define RODIV_PROG	3
#define REFTRIM		0

#define Number_Clicks_Freq_Speed_Up 5
#define Max_Freq_Speed 10000
#define Auto_Revert_Init_Val 400000

#define EEPROM_WR_Delay 10  /* Approx EEPROM write delay in mS */ 

/* Addresses of things in the system... */
#define Write_SPI 0X00

/* Can't believe I have to define this!!! */
#define Pi (double) 3.141592653590
#define sqrt2 (double) 1.414213562373

/* Savy typing and make things clearer in the code */
#define ClearDSPReset PORTD = PORTD | Reset_DSP
#define SetDSPReset PORTD = PORTD & (0xFFFF - Reset_DSP)
#define ClearDACReset PORTD = PORTD | Reset_DAC
#define SetDACReset PORTD = PORTD & (0xFFFF - Reset_DAC)
#define DAC_Reset_Time 20
#define More_Than_1_Audio_Frame_uS 32
#define CselLow DelayUs(1); PORTD = PORTD & (0xFFFF - CSel); DelayUs(1)
#define CselHigh DelayUs(1); PORTD = PORTD | (CSel); DelayUs(1)
#define CselLowPGA2320 DelayUs(1); PORTE = PORTE & (0xFFFF - PGA2320_Select); DelayUs(1)
#define CselHighPGA2320 DelayUs(1); PORTE = PORTE | (PGA2320_Select); DelayUs(1)
#define CselEEPROM DelayUs(1); PORTF = PORTF | (EEPROM_Select); DelayUs(1)
#define CselClearEEPROM DelayUs(1); PORTF = PORTF & (0xFFFF - EEPROM_Select); DelayUs(1)
#define HoldClearEEPROM DelayUs(1); PORTF = PORTF | (EEPROM_Hold); DelayUs(1)
#define HoldEEPROM DelayUs(1); PORTF = PORTF & (0xFFFF - EEPROM_Hold); DelayUs(1)

//#define Write_8_Bits(Data_Word) WriteSPI(Data_Word,8)
//#define Write_16_Bits(Data_Word) WriteSPI(Data_Word,16)
//#define Write_32_Bits(Data_Word) WriteSPI(Data_Word,32)
#define Write_SPI_Addr(Device_Address, Write_Bit) Write_8_Bits(((Device_Address << 1)+Write_Bit))
#define Write_To_SPI 0
#define Read_From_SPI 1
#define Fast_Change_Frequencies 500
#define Fast_Change_Delays 100

/* Old Rotary Encoder = 00XXXXXX*/
#define Encoder_Type_Is_Old (!(PORTB & 0x00C0))
/* Type A rotary Encoder = 10XXXXXX*/
#define Encoder_Type_Is_TypeA (PORTB & Type_A_Rotary_Encoder)
/* Type B Rotary Encoder = 01XXXXXX*/
#define Encoder_Type_Is_TypeB (PORTB & Type_B_Rotary_Encoder)

#define SpeedOfSound 341210   /* millimetres per second */

#define Temp_SPI_Address 7

void	Display_Char(char, char);

void 	Print_Data(int, unsigned char*, unsigned int);

void Load_From_Memory(char );